@@ -43,7 +43,7 @@ type t =
   { common : Jsoo_cmdline.Arg.t
   ; (* compile option *)
     profile : Driver.profile option
-  ; source_map : (string option * Source_map.t) option
+  ; source_map : (string option * Source_map.Standard.t) option
   ; runtime_files : string list
   ; no_runtime : bool
   ; include_runtime : bool
@@ -130,7 +130,7 @@ let options =
   in
   let no_sourcemap =
     let doc =
-      "Don't generate source map. All other source map related flags will be be ignored."
+      "Don't generate source map. All other source map related flags will be ignored."
     in
     Arg.(value & flag & info [ "no-sourcemap"; "no-source-map" ] ~doc)
   in
@@ -280,6 +280,7 @@ let options =
       input_file
       js_files
       keep_unit_names =
+    let inline_source_content = not sourcemap_don't_inline_content in
     let chop_extension s = try Filename.chop_extension s with Invalid_argument _ -> s in
     let runtime_files = js_files in
     let fs_external = fs_external || (toplevel && no_cmis) in
@@ -302,32 +303,18 @@ let options =
       then
         let file, sm_output_file =
           match output_file with
-          | `Name file, _ when sourcemap_inline_in_js -> file, None
-          | `Name file, _ -> file, Some (chop_extension file ^ ".map")
-          | `Stdout, _ -> "STDIN", None
+          | `Name file, _ when sourcemap_inline_in_js -> Some file, None
+          | `Name file, _ -> Some file, Some (chop_extension file ^ ".map")
+          | `Stdout, _ -> None, None
         in
         Some
           ( sm_output_file
-          , { Source_map.version = 3
-            ; file
+          , { (Source_map.Standard.empty ~inline_source_content) with
+              file
             ; sourceroot = sourcemap_root
-            ; sources = []
-            ; sources_content = (if sourcemap_don't_inline_content then None else Some [])
-            ; names = []
-            ; mappings = []
             } )
       else None
     in
-    let source_map =
-      if Option.is_some source_map && not Source_map_io.enabled
-      then (
-        warn
-          "Warning: '--source-map' flag ignored because js_of_ocaml was compiled without \
-           sourcemap support (install yojson to enable support)\n\
-           %!";
-        None)
-      else source_map
-    in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
     let include_dirs = normalize_include_dirs include_dirs in
@@ -408,7 +395,7 @@ let options_runtime_only =
   in
   let no_sourcemap =
     let doc =
-      "Don't generate source map. All other source map related flags will be be ignored."
+      "Don't generate source map. All other source map related flags will be ignored."
     in
     Arg.(value & flag & info [ "no-sourcemap"; "no-source-map" ] ~doc)
   in
@@ -529,6 +516,7 @@ let options_runtime_only =
       target_env
       output_file
       js_files =
+    let inline_source_content = not sourcemap_don't_inline_content in
     let chop_extension s = try Filename.chop_extension s with Invalid_argument _ -> s in
     let runtime_files = js_files in
     let output_file =
@@ -541,32 +529,18 @@ let options_runtime_only =
       then
         let file, sm_output_file =
           match output_file with
-          | `Name file, _ when sourcemap_inline_in_js -> file, None
-          | `Name file, _ -> file, Some (chop_extension file ^ ".map")
-          | `Stdout, _ -> "STDIN", None
+          | `Name file, _ when sourcemap_inline_in_js -> Some file, None
+          | `Name file, _ -> Some file, Some (chop_extension file ^ ".map")
+          | `Stdout, _ -> None, None
         in
         Some
           ( sm_output_file
-          , { Source_map.version = 3
-            ; file
+          , { (Source_map.Standard.empty ~inline_source_content) with
+              file
             ; sourceroot = sourcemap_root
-            ; sources = []
-            ; sources_content = (if sourcemap_don't_inline_content then None else Some [])
-            ; names = []
-            ; mappings = []
             } )
       else None
     in
-    let source_map =
-      if Option.is_some source_map && not Source_map_io.enabled
-      then (
-        warn
-          "Warning: '--source-map' flag ignored because js_of_ocaml was compiled without \
-           sourcemap support (install yojson to enable support)\n\
-           %!";
-        None)
-      else source_map
-    in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
     let include_dirs = normalize_include_dirs include_dirs in
