@@ -21,9 +21,8 @@
 open! Js_of_ocaml_compiler.Stdlib
 open Js_of_ocaml_compiler
 
-let _ =
+let () =
   Sys.catch_break true;
-  Timer.init Sys.time;
   let argv = Jsoo_cmdline.normalize_argv ~warn:(warn "%s") Sys.argv in
   let argv =
     let like_arg x = String.length x > 0 && Char.equal x.[0] '-' in
@@ -40,33 +39,34 @@ let _ =
         then argv
         else
           (* Keep compatibility with js_of_ocaml < 3.6.0 *)
-          Array.of_list
-            (exe :: Cmdliner.Term.name (snd Compile.command) :: maybe_command :: rest)
+          Array.of_list (exe :: Cmdliner.Cmd.name Compile.command :: maybe_command :: rest)
     | _ -> argv
   in
   try
     match
-      Cmdliner.Term.eval_choice
+      Cmdliner.Cmd.eval_value
         ~catch:false
         ~argv
-        Compile.command_main
-        [ Link.command
-        ; Build_fs.command
-        ; Build_runtime.command
-        ; Print_runtime.command
-        ; Check_runtime.command
-        ; Compile.command
-        ]
+        (Cmdliner.Cmd.group
+           ~default:Compile.term
+           (Compile.info "js_of_ocaml")
+           [ Link.command
+           ; Build_fs.command
+           ; Build_runtime.command
+           ; Print_runtime.command
+           ; Check_runtime.command
+           ; Compile.command
+           ])
     with
-    | `Ok () | `Help | `Version ->
+    | Ok (`Ok () | `Help | `Version) ->
         if !warnings > 0 && !werror
         then (
           Format.eprintf "%s: all warnings being treated as errors@." Sys.argv.(0);
           exit 1)
         else exit 0
-    | `Error `Term -> exit 1
-    | `Error `Parse -> exit 124
-    | `Error `Exn -> ()
+    | Error `Term -> exit 1
+    | Error `Parse -> exit Cmdliner.Cmd.Exit.cli_error
+    | Error `Exn -> ()
     (* should not happen *)
   with
   | (Match_failure _ | Assert_failure _ | Not_found) as exc ->
@@ -77,7 +77,7 @@ let _ =
         Sys.argv.(0);
       Format.eprintf "Error: %s@." (Printexc.to_string exc);
       prerr_string backtrace;
-      exit 125
+      exit Cmdliner.Cmd.Exit.internal_error
   | Magic_number.Bad_magic_number s ->
       Format.eprintf "%s: Error: Not an ocaml bytecode file@." Sys.argv.(0);
       Format.eprintf "%s: Error: Invalid magic number %S@." Sys.argv.(0) s;
@@ -109,7 +109,9 @@ let _ =
         comp;
       exit 1
   | Failure s ->
+      let backtrace = Printexc.get_backtrace () in
       Format.eprintf "%s: Error: %s@." Sys.argv.(0) s;
+      prerr_string backtrace;
       exit 1
   | exc ->
       let backtrace = Printexc.get_backtrace () in
