@@ -34,6 +34,10 @@ val to_string : t -> string
 
 val parse : string -> t option
 
+val to_map : t -> string StringMap.t
+
+val of_map : string StringMap.t -> t
+
 val with_kind : t -> kind -> t
 
 exception
