@@ -70,9 +70,10 @@ and mark_expr st e =
   | Apply { f; args; _ } ->
       mark_var st f;
       List.iter args ~f:(fun x -> mark_var st x)
-  | Block (_, a, _) -> Array.iter a ~f:(fun x -> mark_var st x)
-  | Field (x, _) -> mark_var st x
+  | Block (_, a, _, _) -> Array.iter a ~f:(fun x -> mark_var st x)
+  | Field (x, _, _) -> mark_var st x
   | Closure (_, (pc, _)) -> mark_reachable st pc
+  | Special _ -> ()
   | Prim (_, l) ->
       List.iter l ~f:(fun x ->
           match x with
@@ -86,11 +87,11 @@ and mark_reachable st pc =
   then (
     st.reachable_blocks <- Addr.Set.add pc st.reachable_blocks;
     let block = Addr.Map.find pc st.blocks in
-    List.iter block.body ~f:(fun (i, _loc) ->
+    List.iter block.body ~f:(fun i ->
         match i with
         | Let (_, e) -> if not (pure_expr st.pure_funs e) then mark_expr st e
-        | Assign _ -> ()
-        | Set_field (x, _, y) -> (
+        | Event _ | Assign _ -> ()
+        | Set_field (x, _, _, y) -> (
             match st.defs.(Var.idx x) with
             | [ Expr (Block _) ] when st.live.(Var.idx x) = 0 ->
                 (* We will keep this instruction only if x is live *)
@@ -103,7 +104,7 @@ and mark_reachable st pc =
             mark_var st y;
             mark_var st z
         | Offset_ref (x, _) -> mark_var st x);
-    match fst block.branch with
+    match block.branch with
     | Return x | Raise (x, _) -> mark_var st x
     | Stop -> ()
     | Branch cont | Poptrap cont -> mark_cont_reachable st cont
@@ -114,7 +115,7 @@ and mark_reachable st pc =
     | Switch (x, a1) ->
         mark_var st x;
         Array.iter a1 ~f:(fun cont -> mark_cont_reachable st cont)
-    | Pushtrap (cont1, _, cont2, _) ->
+    | Pushtrap (cont1, _, cont2) ->
         mark_cont_reachable st cont1;
         mark_cont_reachable st cont2)
 
@@ -123,8 +124,8 @@ and mark_reachable st pc =
 let live_instr st i =
   match i with
   | Let (x, e) -> st.live.(Var.idx x) > 0 || not (pure_expr st.pure_funs e)
-  | Assign (x, _) | Set_field (x, _, _) -> st.live.(Var.idx x) > 0
-  | Offset_ref _ | Array_set _ -> true
+  | Assign (x, _) | Set_field (x, _, _, _) -> st.live.(Var.idx x) > 0
+  | Event _ | Offset_ref _ | Array_set _ -> true
 
 let rec filter_args st pl al =
   match pl, al with
@@ -142,24 +143,17 @@ let filter_closure blocks st i =
   | Let (x, Closure (l, cont)) -> Let (x, Closure (l, filter_cont blocks st cont))
   | _ -> i
 
-let filter_live_last blocks st (l, loc) =
-  let l =
-    match l with
-    | Return _ | Raise _ | Stop -> l
-    | Branch cont -> Branch (filter_cont blocks st cont)
-    | Cond (x, cont1, cont2) ->
-        Cond (x, filter_cont blocks st cont1, filter_cont blocks st cont2)
-    | Switch (x, a1) ->
-        Switch (x, Array.map a1 ~f:(fun cont -> filter_cont blocks st cont))
-    | Pushtrap (cont1, x, cont2, pcs) ->
-        Pushtrap
-          ( filter_cont blocks st cont1
-          , x
-          , filter_cont blocks st cont2
-          , Addr.Set.inter pcs st.reachable_blocks )
-    | Poptrap cont -> Poptrap (filter_cont blocks st cont)
-  in
-  l, loc
+let filter_live_last blocks st l =
+  match l with
+  | Return _ | Raise _ | Stop -> l
+  | Branch cont -> Branch (filter_cont blocks st cont)
+  | Cond (x, cont1, cont2) ->
+      Cond (x, filter_cont blocks st cont1, filter_cont blocks st cont2)
+  | Switch (x, a1) -> Switch (x, Array.map a1 ~f:(fun cont -> filter_cont blocks st cont))
+  | Pushtrap (cont1, x, cont2) ->
+      Pushtrap (filter_cont blocks st cont1, x, filter_cont blocks st cont2)
+  | Poptrap cont -> Poptrap (filter_cont blocks st cont)
+
 (****)
 
 let ref_count st i =
@@ -173,7 +167,7 @@ let annot st pc xi =
   else
     match (xi : Code.Print.xinstr) with
     | Last _ -> " "
-    | Instr (i, _) ->
+    | Instr i ->
         let c = ref_count st i in
         if c > 0 then Format.sprintf "%d" c else if live_instr st i then " " else "x"
 
@@ -200,19 +194,20 @@ let f ({ blocks; _ } as p : Code.program) =
   let pure_funs = Pure_fun.f p in
   Addr.Map.iter
     (fun _ block ->
-      List.iter block.body ~f:(fun (i, _loc) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (x, e) -> add_def defs x (Expr e)
           | Assign (x, y) -> add_def defs x (Var y)
-          | Set_field (_, _, _) | Array_set (_, _, _) | Offset_ref (_, _) -> ());
-      match fst block.branch with
+          | Event _ | Set_field (_, _, _, _) | Array_set (_, _, _) | Offset_ref (_, _) ->
+              ());
+      match block.branch with
       | Return _ | Raise _ | Stop -> ()
       | Branch cont -> add_cont_dep blocks defs cont
       | Cond (_, cont1, cont2) ->
           add_cont_dep blocks defs cont1;
           add_cont_dep blocks defs cont2
       | Switch (_, a1) -> Array.iter a1 ~f:(fun cont -> add_cont_dep blocks defs cont)
-      | Pushtrap (cont, _, cont_h, _) ->
+      | Pushtrap (cont, _, cont_h) ->
           add_cont_dep blocks defs cont_h;
           add_cont_dep blocks defs cont
       | Poptrap cont -> add_cont_dep blocks defs cont)
@@ -231,10 +226,16 @@ let f ({ blocks; _ } as p : Code.program) =
             pc
             { params = List.filter block.params ~f:(fun x -> st.live.(Var.idx x) > 0)
             ; body =
-                List.filter_map block.body ~f:(fun (i, loc) ->
-                    if live_instr st i
-                    then Some (filter_closure all_blocks st i, loc)
-                    else None)
+                List.fold_left block.body ~init:[] ~f:(fun acc i ->
+                    match i, acc with
+                    | Event _, Event _ :: prev ->
+                        (* Avoid consecutive events (keep just the last one) *)
+                        i :: prev
+                    | _ ->
+                        if live_instr st i
+                        then filter_closure all_blocks st i :: acc
+                        else acc)
+                |> List.rev
             ; branch = filter_live_last all_blocks st block.branch
             }
             blocks)
