@@ -23,6 +23,14 @@ let debug = Debug.find "main"
 
 let times = Debug.find "times"
 
+type optimized_result =
+  { program : Code.program
+  ; variable_uses : Deadcode.variable_uses
+  ; trampolined_calls : Effects.trampolined_calls
+  ; in_cps : Effects.in_cps
+  ; deadcode_sentinal : Code.Var.t
+  }
+
 type profile =
   | O1
   | O2
@@ -87,7 +95,7 @@ let phi p =
 
 let ( +> ) f g x = g (f x)
 
-let map_fst f (x, y) = f x, y
+let map_fst f (x, y, z) = f x, y, z
 
 let effects ~deadcode_sentinal p =
   if Config.Flag.effects ()
@@ -104,9 +112,11 @@ let effects ~deadcode_sentinal p =
         Deadcode.f p
       else p, live_vars
     in
-    let p, cps = p |> Effects.f ~flow_info:info ~live_vars +> map_fst Lambda_lifting.f in
-    p, cps)
-  else p, (Code.Var.Set.empty : Effects.cps_calls)
+    p |> Effects.f ~flow_info:info ~live_vars +> map_fst Lambda_lifting.f)
+  else
+    ( p
+    , (Code.Var.Set.empty : Effects.trampolined_calls)
+    , (Code.Var.Set.empty : Effects.in_cps) )
 
 let exact_calls profile ~deadcode_sentinal p =
   if not (Config.Flag.effects ())
@@ -192,15 +202,14 @@ let generate
     ~exported_runtime
     ~wrap_with_fun
     ~warn_on_unhandled_effect
-    ~deadcode_sentinal
-    ((p, live_vars), cps_calls) =
+    { program; variable_uses; trampolined_calls; deadcode_sentinal; in_cps = _ } =
   if times () then Format.eprintf "Start Generation...@.";
   let should_export = should_export wrap_with_fun in
   Generate.f
-    p
+    program
     ~exported_runtime
-    ~live_vars
-    ~cps_calls
+    ~live_vars:variable_uses
+    ~trampolined_calls
     ~should_export
     ~warn_on_unhandled_effect
     ~deadcode_sentinal
@@ -253,9 +262,9 @@ let gen_missing js missing =
         , ( ECond
               ( EBin
                   ( NotEqEq
-                  , dot (EVar (ident Constant.global_object_)) prim
+                  , dot (EVar (ident Global_constant.global_object_)) prim
                   , EVar (ident_s "undefined") )
-              , dot (EVar (ident Constant.global_object_)) prim
+              , dot (EVar (ident Global_constant.global_object_)) prim
               , EFun
                   ( None
                   , fun_
@@ -378,7 +387,7 @@ let link' ~export_runtime ~standalone ~link (js : Javascript.statement_list) :
                    (EBin
                       ( Eq
                       , dot
-                          (EVar (ident Constant.global_object_))
+                          (EVar (ident Global_constant.global_object_))
                           (Utf8_string.of_string_exn "jsoo_runtime")
                       , EObj all ))
                , N )
@@ -389,7 +398,7 @@ let link' ~export_runtime ~standalone ~link (js : Javascript.statement_list) :
                          (EVar (ident (Utf8_string.of_string_exn "Object")))
                          (Utf8_string.of_string_exn "assign"))
                       [ dot
-                          (EVar (ident Constant.global_object_))
+                          (EVar (ident Global_constant.global_object_))
                           (Utf8_string.of_string_exn "jsoo_runtime")
                       ; EObj all
                       ]
@@ -418,7 +427,7 @@ let link' ~export_runtime ~standalone ~link (js : Javascript.statement_list) :
                          ; rest = None
                          }
                      , ( dot
-                           (EVar (ident Constant.global_object_))
+                           (EVar (ident Global_constant.global_object_))
                            (Utf8_string.of_string_exn "jsoo_runtime")
                        , N ) )
                  ] )
@@ -487,7 +496,7 @@ let coloring js =
 let output formatter ~source_map () js =
   let t = Timer.make () in
   if times () then Format.eprintf "Start Writing file...@.";
-  let sm = Js_output.program formatter ?source_map js in
+  let sm = Js_output.program formatter ~source_map js in
   if times () then Format.eprintf "  write: %a@." Timer.print t;
   sm
 
@@ -524,27 +533,30 @@ let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_
       o#get_free
     in
     let export_shim js =
-      if J.IdentSet.mem (J.ident Constant.exports_) freenames
+      if J.IdentSet.mem (J.ident Global_constant.exports_) freenames
       then
         if should_export wrap_with_fun
-        then var Constant.exports_ (J.EObj []) :: js
+        then var Global_constant.exports_ (J.EObj []) :: js
         else
           let export_node =
             let s =
               Printf.sprintf
                 {|((typeof module === 'object' && module.exports) || %s)|}
-                Constant.global_object
+                Global_constant.global_object
             in
             let lex = Parse_js.Lexer.of_string s in
             Parse_js.parse_expr lex
           in
-          var Constant.exports_ export_node :: js
+          var Global_constant.exports_ export_node :: js
       else js
     in
     let old_global_object_shim js =
-      if J.IdentSet.mem (J.ident Constant.old_global_object_) freenames
+      if J.IdentSet.mem (J.ident Global_constant.old_global_object_) freenames
       then
-        var Constant.old_global_object_ (J.EVar (J.ident Constant.global_object_)) :: js
+        var
+          Global_constant.old_global_object_
+          (J.EVar (J.ident Global_constant.global_object_))
+        :: js
       else js
     in
 
@@ -558,14 +570,15 @@ let pack ~wrap_with_fun ~standalone { Linker.runtime_code = js; always_required_
         then expr (J.EStr (Utf8_string.of_string_exn "use strict")) :: js
         else js
       in
-      f [ J.ident Constant.global_object_ ] js
+      f [ J.ident Global_constant.global_object_ ] js
     in
     match wrap_with_fun with
     | `Anonymous -> expr (mk efun)
     | `Named name ->
         let name = Utf8_string.of_string_exn name in
         mk (sfun (J.ident name))
-    | `Iife -> expr (J.call (mk efun) [ J.EVar (J.ident Constant.global_object_) ] J.N)
+    | `Iife ->
+        expr (J.call (mk efun) [ J.EVar (J.ident Global_constant.global_object_) ] J.N)
   in
   let always_required_js =
     (* consider adding a comments in the generated file with original
@@ -652,16 +665,22 @@ let configure formatter =
   Code.Var.set_pretty (pretty && not (Config.Flag.shortvar ()));
   Code.Var.set_stable (Config.Flag.stable_var ())
 
-let full ~standalone ~wrap_with_fun ~profile ~link ~source_map formatter d p =
-  let exported_runtime = not standalone in
+let link_and_pack ?(standalone = true) ?(wrap_with_fun = `Iife) ?(link = `No) p =
   let export_runtime =
     match link with
     | `All | `All_from _ -> true
     | `Needed | `No -> false
   in
+  p
+  |> link' ~export_runtime ~standalone ~link
+  |> pack ~wrap_with_fun ~standalone
+  |> coloring
+  |> check_js
+
+let optimize ~profile p =
   let deadcode_sentinal =
     (* If deadcode is disabled, this field is just fresh variable *)
-    Code.Var.fresh_n "undef"
+    Code.Var.fresh_n "dummy"
   in
   let opt =
     specialize_js_once
@@ -671,31 +690,31 @@ let full ~standalone ~wrap_with_fun ~profile ~link ~source_map formatter d p =
        | O3 -> o3)
     +> exact_calls ~deadcode_sentinal profile
     +> effects ~deadcode_sentinal
-    +> map_fst (if Config.Flag.effects () then fun x -> x else Generate_closure.f)
+    +> map_fst
+         (match Config.target (), Config.Flag.effects () with
+         | `JavaScript, false -> Generate_closure.f
+         | `JavaScript, true | `Wasm, _ -> Fun.id)
     +> map_fst deadcode'
   in
-  let emit =
-    generate
-      d
-      ~exported_runtime
-      ~wrap_with_fun
-      ~warn_on_unhandled_effect:standalone
-      ~deadcode_sentinal
-    +> link' ~export_runtime ~standalone ~link
-    +> pack ~wrap_with_fun ~standalone
-    +> coloring
-    +> check_js
-    +> output formatter ~source_map ()
-  in
   if times () then Format.eprintf "Start Optimizing...@.";
   let t = Timer.make () in
-  let r = opt p in
+  let (program, variable_uses), trampolined_calls, in_cps = opt p in
   let () = if times () then Format.eprintf " optimizations : %a@." Timer.print t in
-  emit r
+  { program; variable_uses; trampolined_calls; in_cps; deadcode_sentinal }
 
-let full_no_source_map ~standalone ~wrap_with_fun ~profile ~link formatter d p =
-  let (_ : Source_map.t option) =
-    full ~standalone ~wrap_with_fun ~profile ~link ~source_map:None formatter d p
+let full ~standalone ~wrap_with_fun ~profile ~link ~source_map ~formatter d p =
+  let optimized_code = optimize ~profile p in
+  let exported_runtime = not standalone in
+  let emit formatter =
+    generate d ~exported_runtime ~wrap_with_fun ~warn_on_unhandled_effect:standalone
+    +> link_and_pack ~standalone ~wrap_with_fun ~link
+    +> output formatter ~source_map ()
+  in
+  emit formatter optimized_code
+
+let full_no_source_map ~formatter ~standalone ~wrap_with_fun ~profile ~link d p =
+  let (_ : Source_map.info) =
+    full ~standalone ~wrap_with_fun ~profile ~link ~source_map:false ~formatter d p
   in
   ()
 
@@ -704,23 +723,23 @@ let f
     ?(wrap_with_fun = `Iife)
     ?(profile = O1)
     ~link
-    ?source_map
-    formatter
+    ~source_map
+    ~formatter
     d
     p =
-  full ~standalone ~wrap_with_fun ~profile ~link ~source_map formatter d p
+  full ~standalone ~wrap_with_fun ~profile ~link ~source_map ~formatter d p
 
 let f' ?(standalone = true) ?(wrap_with_fun = `Iife) ?(profile = O1) ~link formatter d p =
-  full_no_source_map ~standalone ~wrap_with_fun ~profile ~link formatter d p
+  full_no_source_map ~formatter ~standalone ~wrap_with_fun ~profile ~link d p
 
 let from_string ~prims ~debug s formatter =
   let p, d = Parse_bytecode.from_string ~prims ~debug s in
   full_no_source_map
+    ~formatter
     ~standalone:false
     ~wrap_with_fun:`Anonymous
     ~profile:O1
     ~link:`No
-    formatter
     d
     p
 
