@@ -34,16 +34,25 @@ type def =
   | Expr of Code.expr
   | Param
 
-type info =
-  { info_defs : def array
-  ; info_known_origins : Code.Var.Set.t Code.Var.Tbl.t
-  ; info_maybe_unknown : bool Code.Var.Tbl.t
-  ; info_possibly_mutable : Var.ISet.t
-  }
+module Info = struct
+  type t =
+    { info_defs : def array
+    ; info_known_origins : Code.Var.Set.t Code.Var.Tbl.t
+    ; info_maybe_unknown : bool Code.Var.Tbl.t
+    ; info_possibly_mutable : Var.ISet.t
+    }
 
-let update_def { info_defs; _ } x exp =
-  let idx = Code.Var.idx x in
-  info_defs.(idx) <- Expr exp
+  let def t x =
+    match t.info_defs.(Code.Var.idx x) with
+    | Phi _ | Param -> None
+    | Expr x -> Some x
+
+  let possibly_mutable t x = Code.Var.ISet.mem t.info_possibly_mutable x
+
+  let update_def { info_defs; _ } x exp =
+    let idx = Code.Var.idx x in
+    info_defs.(idx) <- Expr exp
+end
 
 let undefined = Phi Var.Set.empty
 
@@ -90,12 +99,12 @@ let cont_deps blocks vars deps defs (pc, args) =
 
 let expr_deps blocks vars deps defs x e =
   match e with
-  | Constant _ | Apply _ | Prim _ -> ()
+  | Constant _ | Apply _ | Prim _ | Special _ -> ()
   | Closure (l, cont) ->
       List.iter l ~f:(fun x -> add_param_def vars defs x);
       cont_deps blocks vars deps defs cont
-  | Block (_, a, _) -> Array.iter a ~f:(fun y -> add_dep deps x y)
-  | Field (y, _) -> add_dep deps x y
+  | Block (_, a, _, _) -> Array.iter a ~f:(fun y -> add_dep deps x y)
+  | Field (y, _, _) -> add_dep deps x y
 
 let program_deps { blocks; _ } =
   let nv = Var.count () in
@@ -104,7 +113,7 @@ let program_deps { blocks; _ } =
   let defs = Array.make nv undefined in
   Addr.Map.iter
     (fun _ block ->
-      List.iter block.body ~f:(fun (i, _loc) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (x, e) ->
               add_var vars x;
@@ -113,8 +122,8 @@ let program_deps { blocks; _ } =
           | Assign (x, y) ->
               add_dep deps x y;
               add_assign_def vars defs x y
-          | Set_field _ | Array_set _ | Offset_ref _ -> ());
-      match fst block.branch with
+          | Event _ | Set_field _ | Array_set _ | Offset_ref _ -> ());
+      match block.branch with
       | Return _ | Raise _ | Stop -> ()
       | Branch cont | Poptrap cont -> cont_deps blocks vars deps defs cont
       | Cond (_, cont1, cont2) ->
@@ -122,7 +131,7 @@ let program_deps { blocks; _ } =
           cont_deps blocks vars deps defs cont2
       | Switch (_, a1) ->
           Array.iter a1 ~f:(fun cont -> cont_deps blocks vars deps defs cont)
-      | Pushtrap (cont, x, cont_h, _) ->
+      | Pushtrap (cont, x, cont_h) ->
           add_param_def vars defs x;
           cont_deps blocks vars deps defs cont_h;
           cont_deps blocks vars deps defs cont)
@@ -137,12 +146,13 @@ let propagate1 deps defs st x =
   | Phi s -> var_set_lift (fun y -> Var.Tbl.get st y) s
   | Expr e -> (
       match e with
-      | Constant _ | Apply _ | Prim _ | Closure _ | Block _ -> Var.Set.singleton x
-      | Field (y, n) ->
+      | Constant _ | Apply _ | Prim _ | Special _ | Closure _ | Block _ ->
+          Var.Set.singleton x
+      | Field (y, n, _) ->
           var_set_lift
             (fun z ->
               match defs.(Var.idx z) with
-              | Expr (Block (_, a, _)) when n < Array.length a ->
+              | Expr (Block (_, a, _, _)) when n < Array.length a ->
                   let t = a.(n) in
                   add_dep deps x t;
                   Var.Tbl.get st t
@@ -182,15 +192,18 @@ let rec block_escape st x =
       if not (Code.Var.ISet.mem st.may_escape y)
       then (
         Code.Var.ISet.add st.may_escape y;
-        Code.Var.ISet.add st.possibly_mutable y;
         match st.defs.(Var.idx y) with
-        | Expr (Block (_, l, _)) -> Array.iter l ~f:(fun z -> block_escape st z)
-        | _ -> ()))
+        | Expr (Block (_, l, _, mut)) ->
+            (match mut with
+            | Immutable -> ()
+            | Maybe_mutable -> Code.Var.ISet.add st.possibly_mutable y);
+            Array.iter l ~f:(fun z -> block_escape st z)
+        | _ -> Code.Var.ISet.add st.possibly_mutable y))
     (Var.Tbl.get st.known_origins x)
 
 let expr_escape st _x e =
   match e with
-  | Constant _ | Closure _ | Block _ | Field _ -> ()
+  | Special _ | Constant _ | Closure _ | Block _ | Field _ -> ()
   | Apply { args; _ } -> List.iter args ~f:(fun x -> block_escape st x)
   | Prim (Array_get, [ Pv x; _ ]) -> block_escape st x
   | Prim ((Vectlength | Array_get | Not | IsInt | Eq | Neq | Lt | Le | Ult), _) -> ()
@@ -216,15 +229,16 @@ let expr_escape st _x e =
             | Pv v, `Shallow_const -> (
                 match st.defs.(Var.idx v) with
                 | Expr (Constant (Tuple _)) -> ()
-                | Expr (Block (_, a, _)) -> Array.iter a ~f:(fun x -> block_escape st x)
+                | Expr (Block (_, a, _, _)) ->
+                    Array.iter a ~f:(fun x -> block_escape st x)
                 | _ -> block_escape st v)
             | Pv v, `Object_literal -> (
                 match st.defs.(Var.idx v) with
                 | Expr (Constant (Tuple _)) -> ()
-                | Expr (Block (_, a, _)) ->
+                | Expr (Block (_, a, _, _)) ->
                     Array.iter a ~f:(fun x ->
                         match st.defs.(Var.idx x) with
-                        | Expr (Block (_, [| _k; v |], _)) -> block_escape st v
+                        | Expr (Block (_, [| _k; v |], _, _)) -> block_escape st v
                         | Expr (Constant _) -> ()
                         | _ -> block_escape st x)
                 | _ -> block_escape st v)
@@ -239,11 +253,11 @@ let program_escape defs known_origins { blocks; _ } =
   let st = { defs; known_origins; may_escape; possibly_mutable } in
   Addr.Map.iter
     (fun _ block ->
-      List.iter block.body ~f:(fun (i, _loc) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (x, e) -> expr_escape st x e
-          | Assign _ -> ()
-          | Set_field (x, _, y) | Array_set (x, _, y) ->
+          | Event _ | Assign _ -> ()
+          | Set_field (x, _, _, y) | Array_set (x, _, y) ->
               Var.Set.iter
                 (fun y -> Var.ISet.add possibly_mutable y)
                 (Var.Tbl.get known_origins x);
@@ -252,7 +266,7 @@ let program_escape defs known_origins { blocks; _ } =
               Var.Set.iter
                 (fun y -> Var.ISet.add possibly_mutable y)
                 (Var.Tbl.get known_origins x));
-      match fst block.branch with
+      match block.branch with
       | Return x | Raise (x, _) -> block_escape st x
       | Stop | Branch _ | Cond _ | Switch _ | Pushtrap _ | Poptrap _ -> ())
     blocks;
@@ -266,13 +280,13 @@ let propagate2 ?(skip_param = false) defs known_origins possibly_mutable st x =
   | Phi s -> Var.Set.exists (fun y -> Var.Tbl.get st y) s
   | Expr e -> (
       match e with
-      | Constant _ | Closure _ | Apply _ | Prim _ | Block _ -> false
-      | Field (y, n) ->
+      | Constant _ | Closure _ | Apply _ | Prim _ | Block _ | Special _ -> false
+      | Field (y, n, _) ->
           Var.Tbl.get st y
           || Var.Set.exists
                (fun z ->
                  match defs.(Var.idx z) with
-                 | Expr (Block (_, a, _)) ->
+                 | Expr (Block (_, a, _, _)) ->
                      n >= Array.length a
                      || Var.ISet.mem possibly_mutable z
                      || Var.Tbl.get st a.(n)
@@ -295,7 +309,12 @@ let solver2 ?skip_param vars deps defs known_origins possibly_mutable =
   in
   Solver2.f () g (propagate2 ?skip_param defs known_origins possibly_mutable)
 
-let get_approx { info_defs = _; info_known_origins; info_maybe_unknown; _ } f top join x =
+let get_approx
+    { Info.info_defs = _; info_known_origins; info_maybe_unknown; _ }
+    f
+    top
+    join
+    x =
   let s = Var.Tbl.get info_known_origins x in
   if Var.Tbl.get info_maybe_unknown x
   then top
@@ -321,7 +340,37 @@ let the_def_of info x =
         x
   | Pc c -> Some (Constant c)
 
-let the_const_of info x =
+(* If [constant_identical a b = true], then the two values cannot be
+   distinguished, i.e., they are not different objects (and [caml_js_equals a b
+   = true]) and if both are floats, they are bitwise equal. *)
+let constant_identical ~(target : [ `JavaScript | `Wasm ]) a b =
+  match a, b, target with
+  | Int i, Int j, _ -> Targetint.equal i j
+  | Float a, Float b, `JavaScript -> Float.bitwise_equal a b
+  | Float _, Float _, `Wasm -> false
+  | NativeString a, NativeString b, `JavaScript -> Native_string.equal a b
+  | NativeString _, NativeString _, `Wasm ->
+      false
+      (* Native strings are boxed (JavaScript objects) in Wasm and are
+          possibly different objects *)
+  | String a, String b, `JavaScript -> Config.Flag.use_js_string () && String.equal a b
+  | String _, String _, `Wasm ->
+      false (* Strings are boxed in Wasm and are possibly different objects *)
+  | Int32 _, Int32 _, `Wasm ->
+      false (* [Int32]s are boxed in Wasm and are possibly different objects *)
+  | Int32 _, Int32 _, `JavaScript -> assert false
+  | NativeInt _, NativeInt _, `Wasm ->
+      false (* [NativeInt]s are boxed in Wasm and are possibly different objects *)
+  | NativeInt _, NativeInt _, `JavaScript -> assert false
+  (* All other values may be distinct objects and thus different by [caml_js_equals]. *)
+  | Int64 _, Int64 _, _ -> false
+  | Tuple _, Tuple _, _ -> false
+  | Float_array _, Float_array _, _ -> false
+  | (Int _ | Float _ | Int64 _ | Int32 _ | NativeInt _), _, _ -> false
+  | (String _ | NativeString _), _, _ -> false
+  | (Float_array _ | Tuple _), _, _ -> false
+
+let the_const_of ~target info x =
   match x with
   | Pv x ->
       get_approx
@@ -336,30 +385,30 @@ let the_const_of info x =
         None
         (fun u v ->
           match u, v with
-          | Some i, Some j when Poly.(Code.constant_equal i j = Some true) -> u
+          | Some i, Some j when constant_identical ~target i j -> u
           | _ -> None)
         x
   | Pc c -> Some c
 
-let the_int info x =
-  match the_const_of info x with
+let the_int ~target info x =
+  match the_const_of ~target info x with
   | Some (Int i) -> Some i
   | _ -> None
 
-let the_string_of info x =
-  match the_const_of info x with
+let the_string_of ~target info x =
+  match the_const_of info ~target x with
   | Some (String i) -> Some i
   | _ -> None
 
-let the_native_string_of info x =
-  match the_const_of info x with
+let the_native_string_of ~target info x =
+  match the_const_of ~target info x with
   | Some (NativeString i) -> Some i
   | _ -> None
 
 (*XXX Maybe we could iterate? *)
-let direct_approx info x =
+let direct_approx (info : Info.t) x =
   match info.info_defs.(Var.idx x) with
-  | Expr (Field (y, n)) ->
+  | Expr (Field (y, n, _)) ->
       get_approx
         info
         (fun z ->
@@ -367,7 +416,7 @@ let direct_approx info x =
           then None
           else
             match info.info_defs.(Var.idx z) with
-            | Expr (Block (_, a, _)) when n < Array.length a -> Some a.(n)
+            | Expr (Block (_, a, _, _)) when n < Array.length a -> Some a.(n)
             | _ -> None)
         None
         (fun u v ->
@@ -377,7 +426,7 @@ let direct_approx info x =
         y
   | _ -> None
 
-let build_subst info vars =
+let build_subst (info : Info.t) vars =
   let nv = Var.count () in
   let subst = Array.init nv ~f:(fun i -> Var.of_idx i) in
   Var.ISet.iter
@@ -431,7 +480,7 @@ let f ?skip_param p =
       vars;
   let t5 = Timer.make () in
   let info =
-    { info_defs = defs
+    { Info.info_defs = defs
     ; info_known_origins = known_origins
     ; info_maybe_unknown = maybe_unknown
     ; info_possibly_mutable = possibly_mutable
