@@ -34,7 +34,7 @@ let iter_expr_free_vars f e =
       f x;
       List.iter ~f args
   | Block (_, a, _, _) -> Array.iter ~f a
-  | Field (x, _) -> f x
+  | Field (x, _, _) -> f x
   | Closure _ -> ()
   | Special _ -> ()
   | Prim (_, l) ->
@@ -46,7 +46,7 @@ let iter_expr_free_vars f e =
 let iter_instr_free_vars f i =
   match i with
   | Let (_, e) -> iter_expr_free_vars f e
-  | Set_field (x, _, y) ->
+  | Set_field (x, _, _, y) ->
       f x;
       f y
   | Offset_ref (x, _) -> f x
@@ -55,6 +55,7 @@ let iter_instr_free_vars f i =
       f y;
       f z
   | Assign (_, y) -> f y
+  | Event _ -> ()
 
 let iter_last_free_var f l =
   match l with
@@ -73,13 +74,13 @@ let iter_last_free_var f l =
       iter_cont_free_vars f cont2
 
 let iter_block_free_vars f block =
-  List.iter block.body ~f:(fun (i, _) -> iter_instr_free_vars f i);
-  iter_last_free_var f (fst block.branch)
+  List.iter block.body ~f:(fun i -> iter_instr_free_vars f i);
+  iter_last_free_var f block.branch
 
 let iter_instr_bound_vars f i =
   match i with
   | Let (x, _) -> f x
-  | Set_field _ | Offset_ref _ | Array_set _ | Assign _ -> ()
+  | Event _ | Set_field _ | Offset_ref _ | Array_set _ | Assign _ -> ()
 
 let iter_last_bound_vars f l =
   match l with
@@ -88,8 +89,8 @@ let iter_last_bound_vars f l =
 
 let iter_block_bound_vars f block =
   List.iter ~f block.params;
-  List.iter block.body ~f:(fun (i, _) -> iter_instr_bound_vars f i);
-  iter_last_bound_vars f (fst block.branch)
+  List.iter block.body ~f:(fun i -> iter_instr_bound_vars f i);
+  iter_last_bound_vars f block.branch
 
 (****)
 
@@ -100,8 +101,8 @@ type st =
   ; mutable revisited : bool
   }
 
-let find_loops p =
-  let in_loop = ref Addr.Map.empty in
+let find_loops p in_loop pc =
+  let in_loop = ref in_loop in
   let index = ref 0 in
   let state = ref Addr.Map.empty in
   let stack = Stack.create () in
@@ -141,9 +142,17 @@ let find_loops p =
       if st.revisited
       then List.iter !l ~f:(fun pc' -> in_loop := Addr.Map.add pc' pc !in_loop))
   in
-  Code.fold_closures p (fun _ _ (pc, _) () -> traverse pc) ();
+  traverse pc;
   !in_loop
 
+let find_loops_in_closure p pc = find_loops p Addr.Map.empty pc
+
+let find_all_loops p =
+  Code.fold_closures
+    p
+    (fun _ _ (pc, _) (in_loop : _ Addr.Map.t) -> find_loops p in_loop pc)
+    Addr.Map.empty
+
 let mark_variables in_loop p =
   let vars = Var.Tbl.make () (-1) in
   let visited = BitSet.create' p.free_pc in
@@ -156,7 +165,7 @@ let mark_variables in_loop p =
          let pc' = Addr.Map.find pc in_loop in
          iter_block_bound_vars (fun x -> Var.Tbl.set vars x pc') block
        with Not_found -> ());
-      List.iter block.body ~f:(fun (i, _) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (_, Closure (_, (pc', _))) -> traverse pc'
           | _ -> ());
@@ -189,7 +198,7 @@ let free_variables vars in_loop p =
          let pc'' = Addr.Map.find pc in_loop in
          all_freevars := Addr.Map.remove pc'' !all_freevars
        with Not_found -> ());
-      List.iter block.body ~f:(fun (i, _) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (_, Closure (_, (pc', _))) -> (
               traverse pc';
@@ -228,7 +237,7 @@ let f p =
             iter_block_bound_vars (fun x -> Code.Var.ISet.add bound x) block;
             iter_block_free_vars using block;
             List.iter block.body ~f:(function
-                | Let (_, Closure (_, (pc_clo, _))), _ ->
+                | Let (_, Closure (_, (pc_clo, _))) ->
                     Code.Var.Set.iter using (Code.Addr.Map.find pc_clo acc)
                 | _ -> ());
             Code.fold_children p.blocks pc (fun pc' () -> traverse pc') ())
@@ -245,7 +254,7 @@ let f p =
 let f_mutable p =
   Code.invariant p;
   let t = Timer.make () in
-  let in_loop = find_loops p in
+  let in_loop = find_all_loops p in
   let vars = mark_variables in_loop p in
   let free_vars = free_variables vars in_loop p in
   if times () then Format.eprintf "  free vars 1: %a@." Timer.print t;
