@@ -47,7 +47,7 @@ let return_values p =
               { fold = fold_children }
               (fun pc s ->
                 let block = Addr.Map.find pc p.blocks in
-                match fst block.branch with
+                match block.branch with
                 | Return x -> Var.Set.add x s
                 | _ -> s)
               pc
@@ -83,14 +83,14 @@ type escape_status =
 
 type state =
   { vars : Var.ISet.t (* Set of all veriables considered *)
-  ; deps : Var.Set.t array (* Dependency between variables *)
+  ; deps : Var.t Var.Tbl.DataSet.t Var.Tbl.t (* Dependency between variables *)
   ; defs : def array (* Definition of each variable *)
   ; variable_may_escape : escape_status array
         (* Any value bound to this variable may escape *)
-  ; variable_possibly_mutable : bool array
+  ; variable_possibly_mutable : Var.ISet.t
         (* Any value bound to this variable may be mutable *)
   ; may_escape : escape_status array (* This value may escape *)
-  ; possibly_mutable : bool array (* This value may be mutable *)
+  ; possibly_mutable : Var.ISet.t (* This value may be mutable *)
   ; return_values : Var.Set.t Var.Map.t
         (* Set of variables holding return values of each function *)
   ; known_cases : (Var.t, int list) Hashtbl.t
@@ -106,9 +106,7 @@ type state =
 let add_var st x = Var.ISet.add st.vars x
 
 (* x depends on y *)
-let add_dep st x y =
-  let idx = Var.idx y in
-  st.deps.(idx) <- Var.Set.add x st.deps.(idx)
+let add_dep st x y = Var.Tbl.add_set st.deps y x
 
 let add_expr_def st x e =
   add_var st x;
@@ -148,13 +146,22 @@ let cont_deps blocks st ?ignore (pc, args) =
 
 let do_escape st level x = st.variable_may_escape.(Var.idx x) <- level
 
-let possibly_mutable st x = st.variable_possibly_mutable.(Var.idx x) <- true
+let possibly_mutable st x = Var.ISet.add st.variable_possibly_mutable x
 
 let expr_deps blocks st x e =
   match e with
   | Constant _ | Prim ((Vectlength | Not | IsInt | Eq | Neq | Lt | Le | Ult), _) | Block _
     -> ()
-  | Prim ((Extern ("caml_check_bound" | "caml_array_unsafe_get") | Array_get), l) ->
+  | Special _ -> ()
+  | Prim
+      ( ( Extern
+            ( "caml_check_bound"
+            | "caml_check_bound_float"
+            | "caml_check_bound_gen"
+            | "caml_array_unsafe_get"
+            | "caml_floatarray_unsafe_get" )
+        | Array_get )
+      , l ) ->
       (* The analysis knowns about these primitives, and will compute
          an approximation of the value they return based on an
          approximation of their arguments *)
@@ -193,15 +200,15 @@ let expr_deps blocks st x e =
             | Pv v, `Const -> do_escape st Escape_constant v
             | Pv v, `Shallow_const -> (
                 match st.defs.(Var.idx v) with
-                | Expr (Block (_, a, _)) ->
+                | Expr (Block (_, a, _, _)) ->
                     Array.iter a ~f:(fun x -> do_escape st Escape x)
                 | _ -> do_escape st Escape v)
             | Pv v, `Object_literal -> (
                 match st.defs.(Var.idx v) with
-                | Expr (Block (_, a, _)) ->
+                | Expr (Block (_, a, _, _)) ->
                     Array.iter a ~f:(fun x ->
                         match st.defs.(Var.idx x) with
-                        | Expr (Block (_, [| _k; v |], _)) -> do_escape st Escape v
+                        | Expr (Block (_, [| _k; v |], _, _)) -> do_escape st Escape v
                         | _ -> do_escape st Escape x)
                 | _ -> do_escape st Escape v)
             | Pv v, `Mutable -> do_escape st Escape v);
@@ -224,22 +231,31 @@ let expr_deps blocks st x e =
   | Closure (l, cont) ->
       List.iter l ~f:(fun x -> add_param_def st x);
       cont_deps blocks st cont
-  | Field (y, _) -> add_dep st x y
+  | Field (y, _, _) -> add_dep st x y
 
-let program_deps st { blocks; _ } =
+let program_deps st { start; blocks; _ } =
+  Code.traverse
+    { Code.fold = Code.fold_children }
+    (fun pc () ->
+      match Addr.Map.find pc blocks with
+      | { branch = Return x; _ } -> do_escape st Escape x
+      | _ -> ())
+    start
+    blocks
+    ();
   Addr.Map.iter
     (fun _ block ->
-      List.iter block.body ~f:(fun (i, _) ->
+      List.iter block.body ~f:(fun i ->
           match i with
           | Let (x, e) ->
               add_expr_def st x e;
               expr_deps blocks st x e
           | Assign (x, y) -> add_assign_def st x y
-          | Set_field (x, _, y) | Array_set (x, _, y) ->
+          | Set_field (x, _, _, y) | Array_set (x, _, y) ->
               possibly_mutable st x;
               do_escape st Escape y
-          | Offset_ref _ -> ());
-      match fst block.branch with
+          | Event _ | Offset_ref _ -> ());
+      match block.branch with
       | Return _ | Stop -> ()
       | Raise (x, _) -> do_escape st Escape x
       | Branch cont | Poptrap cont -> cont_deps blocks st cont
@@ -266,15 +282,15 @@ let program_deps st { blocks; _ } =
                   (fun pc tags ->
                     let block = Addr.Map.find pc blocks in
                     List.iter
-                      ~f:(fun (i, _) ->
+                      ~f:(fun i ->
                         match i with
-                        | Let (y, Field (x', _)) when Var.equal b x' ->
+                        | Let (y, Field (x', _, _)) when Var.equal b x' ->
                             Hashtbl.add st.known_cases y tags
                         | _ -> ())
                       block.body)
                   h
             | Expr _ | Phi _ -> ())
-      | Pushtrap (cont, x, cont_h, _) ->
+      | Pushtrap (cont, x, cont_h) ->
           add_var st x;
           st.defs.(Var.idx x) <- Phi { known = Var.Set.empty; others = true };
           cont_deps blocks st cont_h;
@@ -324,12 +340,13 @@ module Domain = struct
     then (
       st.may_escape.(idx) <- s;
       match st.defs.(idx) with
-      | Expr (Block (_, a, _)) ->
+      | Expr (Block (_, a, _, mut)) -> (
           Array.iter ~f:(fun y -> variable_escape ~update ~st ~approx s y) a;
-          if Poly.equal s Escape
-          then (
-            st.possibly_mutable.(idx) <- true;
-            update ~children:true x)
+          match s, mut with
+          | Escape, Maybe_mutable ->
+              Var.ISet.add st.possibly_mutable x;
+              update ~children:true x
+          | (Escape_constant | No), _ | Escape, Immutable -> ())
       | Expr (Closure (params, _)) ->
           List.iter
             ~f:(fun y ->
@@ -378,10 +395,14 @@ module Domain = struct
     | Values { known; _ } ->
         Var.Set.iter
           (fun x ->
-            if not st.possibly_mutable.(Var.idx x)
-            then (
-              st.possibly_mutable.(Var.idx x) <- true;
-              update ~children:true x))
+            match st.defs.(Var.idx x) with
+            | Expr (Block (_, _, _, Maybe_mutable)) ->
+                if not (Var.ISet.mem st.possibly_mutable x)
+                then (
+                  Var.ISet.add st.possibly_mutable x;
+                  update ~children:true x)
+            | Expr (Block (_, _, _, Immutable)) | Expr (Closure _) -> ()
+            | Phi _ | Expr _ -> assert false)
           known
 end
 
@@ -395,7 +416,7 @@ let propagate st ~update approx x =
           (* A constant cannot contain a function *)
           Domain.bot
       | Closure _ | Block _ -> Domain.singleton x
-      | Field (y, n) -> (
+      | Field (y, n, _) -> (
           match Var.Tbl.get approx y with
           | Values { known; others } ->
               let tags =
@@ -408,7 +429,7 @@ let propagate st ~update approx x =
                 ~approx
                 (fun z ->
                   match st.defs.(Var.idx z) with
-                  | Expr (Block (t, a, _))
+                  | Expr (Block (t, a, _, _))
                     when n < Array.length a
                          &&
                          match tags with
@@ -417,15 +438,19 @@ let propagate st ~update approx x =
                       let t = a.(n) in
                       add_dep st x t;
                       let a = Var.Tbl.get approx t in
-                      if st.possibly_mutable.(Var.idx z)
+                      if Var.ISet.mem st.possibly_mutable z
                       then Domain.join ~update ~st ~approx Domain.others a
                       else a
                   | Expr (Block _ | Closure _) -> Domain.bot
                   | Phi _ | Expr _ -> assert false)
                 known
           | Top -> Top)
-      | Prim (Extern "caml_check_bound", [ Pv y; _ ]) -> Var.Tbl.get approx y
-      | Prim ((Array_get | Extern "caml_array_unsafe_get"), [ Pv y; _ ]) -> (
+      | Prim
+          ( Extern ("caml_check_bound" | "caml_check_bound_float" | "caml_check_bound_gen")
+          , [ Pv y; _ ] ) -> Var.Tbl.get approx y
+      | Prim
+          ( (Array_get | Extern ("caml_array_unsafe_get" | "caml_floatarray_unsafe_get"))
+          , [ Pv y; _ ] ) -> (
           if st.fast
           then Domain.others
           else
@@ -438,7 +463,7 @@ let propagate st ~update approx x =
                   ~others
                   (fun z ->
                     match st.defs.(Var.idx z) with
-                    | Expr (Block (_, lst, _)) ->
+                    | Expr (Block (_, lst, _, _)) ->
                         Array.iter ~f:(fun t -> add_dep st x t) lst;
                         let a =
                           Array.fold_left
@@ -447,7 +472,7 @@ let propagate st ~update approx x =
                             ~init:Domain.bot
                             lst
                         in
-                        if st.possibly_mutable.(Var.idx z)
+                        if Var.ISet.mem st.possibly_mutable z
                         then Domain.join ~update ~st ~approx Domain.others a
                         else a
                     | Expr (Closure _) -> Domain.bot
@@ -460,6 +485,7 @@ let propagate st ~update approx x =
              block *)
           Domain.bot
       | Prim (Extern _, _) -> Domain.others
+      | Special _ -> Domain.others
       | Apply { f; args; _ } -> (
           match Var.Tbl.get approx f with
           | Values { known; others } ->
@@ -534,7 +560,8 @@ let propagate st ~update approx x =
       (match st.variable_may_escape.(Var.idx x) with
       | (Escape | Escape_constant) as s -> Domain.approx_escape ~update ~st ~approx s res
       | No -> ());
-      if st.variable_possibly_mutable.(Var.idx x) then Domain.mark_mutable ~update ~st res;
+      if Var.ISet.mem st.variable_possibly_mutable x
+      then Domain.mark_mutable ~update ~st res;
       res
   | Top -> Top
 
@@ -544,7 +571,8 @@ module Solver = G.Solver (Domain)
 let solver st =
   let g =
     { G.domain = st.vars
-    ; G.iter_children = (fun f x -> Var.Set.iter f st.deps.(Var.idx x))
+    ; G.iter_children =
+        (fun f x -> Var.Tbl.DataSet.iter (fun k -> f k) (Var.Tbl.get st.deps x))
     }
   in
   Solver.f' () g (propagate st)
@@ -565,12 +593,12 @@ let f ~fast p =
   let rets = return_values p in
   let nv = Var.count () in
   let vars = Var.ISet.empty () in
-  let deps = Array.make nv Var.Set.empty in
+  let deps = Var.Tbl.make_set () in
   let defs = Array.make nv undefined in
   let variable_may_escape = Array.make nv No in
-  let variable_possibly_mutable = Array.make nv false in
+  let variable_possibly_mutable = Var.ISet.empty () in
   let may_escape = Array.make nv No in
-  let possibly_mutable = Array.make nv false in
+  let possibly_mutable = Var.ISet.empty () in
   let st =
     { vars
     ; deps
@@ -621,17 +649,17 @@ let f ~fast p =
                            x
                            (match st.defs.(Var.idx x) with
                            | Expr (Closure _) -> "C"
-                           | Expr (Block _) ->
+                           | Expr (Block _) -> (
                                "B"
                                ^
-                               if Poly.equal st.may_escape.(Var.idx x) Escape
-                               then "X"
-                               else ""
+                               match st.may_escape.(Var.idx x) with
+                               | Escape -> "X"
+                               | _ -> "")
                            | _ -> "O")))
                     (Var.Set.elements known)
                     others
-                    st.possibly_mutable.(Var.idx x)
-                    st.variable_possibly_mutable.(Var.idx x)
+                    (Var.ISet.mem st.possibly_mutable x)
+                    (Var.ISet.mem st.variable_possibly_mutable x)
                     (match st.variable_may_escape.(Var.idx x) with
                     | Escape -> "Y"
                     | Escape_constant -> "y"
