@@ -21,7 +21,8 @@
 open Javascript
 
 val if_statement :
-     expression
+     function_end:(unit -> location)
+  -> expression
   -> location
   -> statement * location
   -> bool
@@ -32,3 +33,5 @@ val if_statement :
 val block : (Javascript.statement * location) list -> Javascript.statement * location
 
 val unblock : Javascript.statement * location -> (Javascript.statement * location) list
+
+val function_body : (statement * location) list -> (statement * location) list
