@@ -52,6 +52,8 @@ class type mapper = object
     -> Javascript.for_binding
     -> Javascript.for_binding
 
+  method binding_property : Javascript.binding_property -> Javascript.binding_property
+
   method variable_declaration :
        Javascript.variable_declaration_kind
     -> Javascript.variable_declaration
@@ -185,7 +187,7 @@ class map : mapper =
           ForAwaitOf_statement (e1, m#expression e2, (m#statement s, m#loc loc))
       | Continue_statement s -> Continue_statement s
       | Break_statement s -> Break_statement s
-      | Return_statement e -> Return_statement (m#expression_o e)
+      | Return_statement (e, loc) -> Return_statement (m#expression_o e, m#loc loc)
       | Labelled_statement (l, (s, loc)) ->
           Labelled_statement (l, (m#statement s, m#loc loc))
       | Throw_statement e -> Throw_statement (m#expression e)
@@ -295,10 +297,11 @@ class map : mapper =
               EAssignTarget
                 (ObjectTarget
                    (List.map l ~f:(function
-                       | TargetPropertyId (i, e) ->
-                           TargetPropertyId (m#ident i, m#initialiser_o e)
-                       | TargetProperty (i, e) ->
-                           TargetProperty (m#property_name i, m#expression e)
+                       | TargetPropertyId (Prop_and_ident i, e) ->
+                           TargetPropertyId (Prop_and_ident (m#ident i), m#initialiser_o e)
+                       | TargetProperty (n, e, i) ->
+                           TargetProperty
+                             (m#property_name n, m#expression e, m#initialiser_o i)
                        | TargetPropertyMethod (n, x) ->
                            TargetPropertyMethod (m#property_name n, m#method_ x)
                        | TargetPropertySpread e -> TargetPropertySpread (m#expression e))))
@@ -311,8 +314,8 @@ class map : mapper =
       | EAccess (e1, ak, e2) -> EAccess (m#expression e1, ak, m#expression e2)
       | EDot (e1, ak, id) -> EDot (m#expression e1, ak, id)
       | EDotPrivate (e1, ak, id) -> EDotPrivate (m#expression e1, ak, id)
-      | ENew (e1, args) ->
-          ENew (m#expression e1, Option.map ~f:(List.map ~f:m#argument) args)
+      | ENew (e1, args, loc) ->
+          ENew (m#expression e1, Option.map ~f:(List.map ~f:m#argument) args, m#loc loc)
       | EVar v -> EVar (m#ident v)
       | EFun (idopt, fun_decl) ->
           let idopt = Option.map ~f:m#ident idopt in
@@ -376,10 +379,11 @@ class map : mapper =
       | None -> None
       | Some (b, e) -> Some (m#binding b, m#initialiser_o e)
 
-    method private binding_property x =
+    method binding_property x =
       match x with
       | Prop_binding (i, e) -> Prop_binding (m#property_name i, m#binding_element e)
-      | Prop_ident (i, e) -> Prop_ident (m#ident i, m#initialiser_o e)
+      | Prop_ident (Prop_and_ident i, e) ->
+          Prop_ident (Prop_and_ident (m#ident i), m#initialiser_o e)
 
     method expression_o x =
       match x with
@@ -545,7 +549,7 @@ class iter : iterator =
           m#statement s
       | Continue_statement _ -> ()
       | Break_statement _ -> ()
-      | Return_statement e -> m#expression_o e
+      | Return_statement (e, _) -> m#expression_o e
       | Labelled_statement (_, (s, _)) -> m#statement s
       | Throw_statement e -> m#expression e
       | Switch_statement (e, l, def, l') ->
@@ -641,12 +645,13 @@ class iter : iterator =
                   | TargetElementSpread e -> m#expression e)
           | ObjectTarget l ->
               List.iter l ~f:(function
-                  | TargetPropertyId (i, e) ->
+                  | TargetPropertyId (Prop_and_ident i, e) ->
                       m#ident i;
                       m#initialiser_o e
-                  | TargetProperty (i, e) ->
-                      m#property_name i;
-                      m#expression e
+                  | TargetProperty (n, e, i) ->
+                      m#property_name n;
+                      m#expression e;
+                      m#initialiser_o i
                   | TargetPropertyMethod (n, x) ->
                       m#property_name n;
                       m#method_ x
@@ -663,10 +668,10 @@ class iter : iterator =
           m#expression e2
       | EDot (e1, _ak, _) -> m#expression e1
       | EDotPrivate (e1, _, _) -> m#expression e1
-      | ENew (e1, Some args) ->
+      | ENew (e1, Some args, _) ->
           m#expression e1;
           List.iter args ~f:m#argument
-      | ENew (e1, None) -> m#expression e1
+      | ENew (e1, None, _) -> m#expression e1
       | EVar v -> m#ident v
       | EFun (idopt, fun_decl) ->
           (match idopt with
@@ -736,7 +741,7 @@ class iter : iterator =
     method private binding_property x =
       match x with
       | Prop_binding ((_ : property_name), e) -> m#binding_element e
-      | Prop_ident (i, e) ->
+      | Prop_ident (Prop_and_ident i, e) ->
           m#ident i;
           m#initialiser_o e
 
@@ -1054,7 +1059,7 @@ class free =
           let body = tbody#function_body body in
           let params = tbody#formal_parameter_list params in
           tbody#record_block (Params params);
-          m#def_var id;
+          m#def_local id;
           m#merge_info tbody;
           Function_declaration (id, (k, params, body, nid))
       | Class_declaration (id, cl_decl) ->
@@ -1063,7 +1068,7 @@ class free =
           let cl_decl = cbody#class_decl cl_decl in
           cbody#record_block Normal;
           m#merge_block_info cbody;
-          m#def_var id;
+          m#def_local id;
           Class_declaration (id, cl_decl)
       | Block b -> Block (m#block b)
       | For_statement (Right (((Const | Let) as k), l), e1, e2, (st, loc)) ->
@@ -1340,6 +1345,14 @@ class rename_variable ~esm =
         let m' = m#update_state Lexical_block [] p in
         m'#statements p
 
+    method binding_property x =
+      match x with
+      | Prop_ident (Prop_and_ident (S { name = Utf8 name' as name; _ } as ident), e)
+        when StringMap.mem name' subst ->
+          let x = Prop_binding (PNI name, (BindingIdent ident, e)) in
+          super#binding_property x
+      | x -> super#binding_property x
+
     method expression e =
       match e with
       | EFun (ident, (k, params, body, nid)) ->
@@ -1351,6 +1364,16 @@ class rename_variable ~esm =
       | EClass (Some id, cl_decl) ->
           let m' = m#update_state Lexical_block [ id ] [] in
           EClass (Some (m'#ident id), m'#class_decl cl_decl)
+      | EAssignTarget (ObjectTarget l) ->
+          let l =
+            List.map l ~f:(function
+                | TargetPropertyId
+                    (Prop_and_ident (S { name = Utf8 name' as name; _ } as ident), rhs)
+                  when StringMap.mem name' subst ->
+                    TargetProperty (PNI name, EVar ident, rhs)
+                | b -> b)
+          in
+          super#expression (EAssignTarget (ObjectTarget l))
       | _ -> super#expression e
 
     method statement s =
@@ -1477,78 +1500,20 @@ class rename_variable ~esm =
   end
 
 class compact_vardecl =
+  let expr_eq id e = EBin (Eq, EVar id, e) in
   object (m)
-    inherit free as super
-
-    val mutable exc_ = IdentSet.empty
+    inherit map as super
 
     val mutable insert_ = IdentSet.empty
 
-    method exc = exc_
+    method private var x = insert_ <- IdentSet.add x insert_
 
-    method private translate l =
-      List.filter_map l ~f:(function
-          | DeclPattern _ -> None
-          | DeclIdent (id, eopt) -> (
-              match eopt with
-              | None -> None
-              | Some (e, _) -> Some (EBin (Eq, EVar id, e))))
-
-    method private translate_st l =
-      let l = m#translate l in
-      match l with
-      | [] -> Empty_statement
-      | x :: l ->
-          Expression_statement (List.fold_left l ~init:x ~f:(fun acc e -> ESeq (acc, e)))
-
-    method private translate_ex l =
-      let l = m#translate l in
-      match l with
-      | [] -> None
-      | x :: l -> Some (List.fold_left l ~init:x ~f:(fun acc e -> ESeq (acc, e)))
-
-    method private except_ids l =
-      exc_ <- List.fold_left l ~init:exc_ ~f:(fun acc s -> IdentSet.add s acc)
-
-    method private except_ident e = exc_ <- IdentSet.add e exc_
-
-    method statement s =
-      let s = super#statement s in
-      match s with
-      | Function_declaration (id, fun_decl) ->
-          let fun_decl = m#fun_decl fun_decl in
-          m#except_ident id;
-          Function_declaration (id, fun_decl)
-      | Variable_statement (_, l) -> m#translate_st l
-      | For_statement (Right (Var, l), e2, e3, s) ->
-          For_statement (Left (m#translate_ex l), e2, e3, s)
-      | Try_statement (b, w, f) ->
-          (match w with
-          | None -> ()
-          | Some (None, _) -> ()
-          | Some (Some (id, _), _) ->
-              let ids = bound_idents_of_binding id in
-              m#except_ids ids);
-          Try_statement (b, w, f)
-      | s -> s
-
-    method record_block block =
-      (match block with
-      | Catch (id, _) ->
-          let ids = bound_idents_of_binding id in
-          m#except_ids ids
-      | Params p ->
-          let s = bound_idents_of_params p in
-          m#except_ids s
-      | Normal -> ());
-      super#record_block block
-
-    method merge_info from =
-      super#merge_info from;
-      let all =
-        IdentSet.fold (fun e acc -> IdentSet.add e acc) from#state.def_var IdentSet.empty
-      in
-      insert_ <- IdentSet.diff all from#exc
+    method fun_decl (k, params, body, nid) =
+      let m' = {<insert_ = IdentSet.empty>} in
+      let params = m'#formal_parameter_list params in
+      let body = m'#function_body body in
+      let body = m'#pack body in
+      k, params, body, m#loc nid
 
     method private split x =
       let rec loop = function
@@ -1557,14 +1522,15 @@ class compact_vardecl =
       in
       loop x
 
-    method private pack (all : IdentSet.t) sources =
+    method pack rest =
+      let all = insert_ in
       let may_flush rem vars s instr =
         if List.is_empty vars
         then rem, [], s :: instr
         else rem, [], s :: (Variable_statement (Var, List.rev vars), N) :: instr
       in
       let rem, vars, instr =
-        List.fold_left sources ~init:(all, [], []) ~f:(fun (rem, vars, instr) (s, loc) ->
+        List.fold_left rest ~init:(all, [], []) ~f:(fun (rem, vars, instr) (s, loc) ->
             match s with
             | Expression_statement e ->
                 let l = m#split e in
@@ -1592,41 +1558,24 @@ class compact_vardecl =
           (Variable_statement (Var, List.rev_append l l'), loc) :: rest
       | l, _ -> (Variable_statement (Var, l), N) :: instr
 
-    method fun_decl (k, params, body, nid) =
-      let all = IdentSet.diff insert_ exc_ in
-      let body = m#pack all body in
-      k, params, body, nid
-
-    method expression x =
-      let x = super#expression x in
-      match x with
-      | EFun (ident, fun_decl) ->
-          let fun_decl = m#fun_decl fun_decl in
-          Option.iter ~f:m#except_ident ident;
-          EFun (ident, fun_decl)
-      | _ -> x
-
-    method statements l =
-      let l = super#statements l in
-      let l =
-        List.fold_left l ~init:[] ~f:(fun acc (x, loc) ->
-            match x with
-            | Expression_statement e ->
-                let l = m#split e in
-                let l =
-                  List.fold_left l ~init:acc ~f:(fun acc e ->
-                      (Expression_statement e, N) :: acc)
-                in
-                l
-            | _ -> (x, loc) :: acc)
-      in
-      List.rev l
+    method statements s =
+      let s = super#statements s in
+      List.concat_map s ~f:(fun (s, loc) ->
+          match s with
+          | Variable_statement (Var, l) ->
+              List.filter_map l ~f:(function
+                  | DeclIdent (x, Some (init, loc)) ->
+                      m#var x;
+                      Some (Expression_statement (expr_eq x init), loc)
+                  | DeclIdent (x, None) ->
+                      m#var x;
+                      None
+                  | DeclPattern _ as x -> Some (Variable_statement (Var, [ x ]), loc))
+          | s -> [ s, loc ])
 
     method program p =
       let p = super#program p in
-      m#merge_info m;
-      let all = IdentSet.diff insert_ exc_ in
-      let body = m#pack all p in
+      let body = m#pack p in
       body
   end
 
@@ -1757,7 +1706,12 @@ class simpl =
       | EFun
           (None, (({ generator = false; async = true | false }, _, body, _) as fun_decl))
         when Config.Flag.es6 () && not (use_fun_context body) ->
-          EArrow (fun_decl, false, ANo_fun_context)
+          let consise =
+            match body with
+            | [ (Return_statement _, _) ] -> true
+            | _ -> false
+          in
+          EArrow (fun_decl, consise, ANo_fun_context)
       | EArrow (((_, _, body, _) as fun_decl), consise, AUnknown) ->
           if use_fun_context body
           then EArrow (fun_decl, consise, AUse_parent_fun_context)
@@ -1790,17 +1744,41 @@ class simpl =
       let s = super#statements s in
       List.fold_right s ~init:[] ~f:(fun (st, loc) rem ->
           match st with
+          (* if (1) e1 ... --> e1 *)
           | If_statement (ENum n, iftrue, _) when Num.is_one n -> iftrue :: rem
+          (* if (0) e1 else e2 --> e2 *)
           | If_statement (ENum n, _, iffalse) when Num.is_zero n -> opt_cons iffalse rem
+          (* if (e1) return e2 else return e3 --> return e1 ? e2 : e3 *)
           | If_statement
-              (cond, (Return_statement (Some e1), _), Some (Return_statement (Some e2), _))
-            -> (Return_statement (Some (ECond (cond, e1, e2))), loc) :: rem
+              ( cond
+              , (Return_statement (Some e1, _), _)
+              , Some (Return_statement (Some e2, _), _) ) ->
+              ( Return_statement
+                  ( Some (ECond (cond, e1, e2))
+                  , U
+                    (*TODO: it would be better to use the location of the
+                      end of the function, but we can't easily get it. *) )
+              , loc )
+              :: rem
+          (* if (e1) v1 = e2 else v1 = e3 --> v1 = e1 ? e2 : e3 *)
           | If_statement
               ( cond
               , (Expression_statement (EBin (Eq, v1, e1)), _)
               , Some (Expression_statement (EBin (Eq, v2, e2)), _) )
             when Poly.(v1 = v2) ->
               (Expression_statement (EBin (Eq, v1, ECond (cond, e1, e2))), loc) :: rem
+          (* The following optimizations cause the generated JS to compress less.
+             (* if (e1) e2 else e3 --> e1 ? e2 : e3 *)
+             | If_statement
+                 (e1, (Expression_statement e2, _), Some (Expression_statement e3, _)) ->
+                 (Expression_statement (ECond (e1, e2, e3)), loc) :: rem
+             (* if (!e1) e2 --> e1 || e2 *)
+             | If_statement (EUn (Not, e1), (Expression_statement e2, _), None) ->
+                 (Expression_statement (EBin (Or, e1, e2)), loc) :: rem
+             (* if (e1) e2 --> e1 && e2 *)
+             | If_statement (e1, (Expression_statement e2, _), None) ->
+                 (Expression_statement (EBin (And, e1, e2)), loc) :: rem
+          *)
           | Variable_statement (((Var | Let | Const) as k), l1) ->
               let x = List.map l1 ~f:(fun d -> Variable_statement (k, [ d ]), loc) in
               x @ rem
