@@ -134,18 +134,15 @@ module Check = struct
     in
     let freename = StringSet.diff freename Reserved.keyword in
     let freename = StringSet.diff freename Reserved.provided in
-    let freename = StringSet.remove Constant.global_object freename in
+    let freename = StringSet.remove Global_constant.global_object freename in
     let freename = if has_flags then StringSet.remove "FLAG" freename else freename in
-    if StringSet.mem Constant.old_global_object freename && false
-       (* Don't warn yet, we want to give a transition period where both
-          "globalThis" and "joo_global_object" are allowed without extra
-          noise *)
+    if StringSet.mem Global_constant.old_global_object freename
     then
       warn
         "warning: %s: 'joo_global_object' is being deprecated, please use `globalThis` \
          instead@."
         (loc pi);
-    let freename = StringSet.remove Constant.old_global_object freename in
+    let freename = StringSet.remove Global_constant.old_global_object freename in
     let defname = to_stringset free#get_def in
     if not (StringSet.mem name defname)
     then
@@ -177,12 +174,25 @@ module Fragment = struct
     ; weakdef : bool
     ; always : bool
     ; code : Javascript.program pack
-    ; js_string : bool option
-    ; effects : bool option
+    ; conditions : bool StringMap.t
     ; fragment_target : Target_env.t option
     ; aliases : StringSet.t
+    ; deprecated : string option
     }
 
+  let allowed_flags =
+    List.fold_left
+      ~f:(fun m (k, v) -> StringMap.add k v m)
+      ~init:StringMap.empty
+      [ "js-string", Config.Flag.use_js_string
+      ; "effects", Config.Flag.effects
+      ; ( "wasm"
+        , fun () ->
+            match Config.target () with
+            | `JavaScript -> false
+            | `Wasm -> true )
+      ]
+
   type t =
     | Always_include of Javascript.program pack
     | Fragment of fragment_
@@ -247,10 +257,10 @@ module Fragment = struct
                 ; always = false
                 ; has_macro = false
                 ; code = Ok code
-                ; js_string = None
-                ; effects = None
+                ; conditions = StringMap.empty
                 ; fragment_target = None
                 ; aliases = StringSet.empty
+                ; deprecated = None
                 }
               in
               let fragment =
@@ -281,31 +291,25 @@ module Fragment = struct
                     | `Always -> { fragment with always = true }
                     | `Alias name ->
                         { fragment with aliases = StringSet.add name fragment.aliases }
-                    | (`Ifnot "js-string" | `If "js-string") as i ->
-                        let b =
-                          match i with
-                          | `If _ -> true
-                          | `Ifnot _ -> false
-                        in
-                        if Option.is_some fragment.js_string
-                        then Format.eprintf "Duplicated js-string in %s\n" (loc pi);
-                        { fragment with js_string = Some b }
-                    | (`Ifnot "effects" | `If "effects") as i ->
-                        let b =
-                          match i with
-                          | `If _ -> true
-                          | `Ifnot _ -> false
-                        in
-                        if Option.is_some fragment.effects
-                        then Format.eprintf "Duplicated effects in %s\n" (loc pi);
-                        { fragment with effects = Some b }
+                    | `Deprecated txt -> { fragment with deprecated = Some txt }
                     | `If name when Option.is_some (Target_env.of_string name) ->
                         if Option.is_some fragment.fragment_target
                         then Format.eprintf "Duplicated target_env in %s\n" (loc pi);
                         { fragment with fragment_target = Target_env.of_string name }
-                    | `If name | `Ifnot name ->
-                        Format.eprintf "Unkown flag %S in %s\n" name (loc pi);
-                        fragment)
+                    | (`Ifnot v | `If v) when not (StringMap.mem v allowed_flags) ->
+                        Format.eprintf "Unkown flag %S in %s\n" v (loc pi);
+                        fragment
+                    | (`Ifnot v | `If v) as i ->
+                        if StringMap.mem v fragment.conditions
+                        then Format.eprintf "Duplicated %s in %s\n" v (loc pi);
+                        let b =
+                          match i with
+                          | `If _ -> true
+                          | `Ifnot _ -> false
+                        in
+                        { fragment with
+                          conditions = StringMap.add v b fragment.conditions
+                        })
               in
               Fragment fragment)
     in
@@ -393,6 +397,9 @@ type state =
   { ids : IntSet.t
   ; always_required_codes : always_required list
   ; codes : (Javascript.program pack * bool) list
+  ; deprecation : (int list * string) list
+  ; missing : StringSet.t
+  ; include_ : string -> bool
   }
 
 type output =
@@ -403,6 +410,7 @@ type output =
 type provided =
   { id : int
   ; pi : Parse_info.t
+  ; filename : string
   ; weakdef : bool
   ; target_env : Target_env.t
   }
@@ -423,6 +431,17 @@ let reset () =
   Primitive.reset ();
   Generate.init ()
 
+let list_all ?from () =
+  let include_ =
+    match from with
+    | None -> fun _ _ -> true
+    | Some l -> fun fn _nm -> List.mem fn ~set:l
+  in
+  Hashtbl.fold
+    (fun nm p set -> if include_ p.filename nm then StringSet.add nm set else set)
+    provided
+    StringSet.empty
+
 let load_fragment ~target_env ~filename (f : Fragment.t) =
   match f with
   | Always_include code ->
@@ -437,25 +456,19 @@ let load_fragment ~target_env ~filename (f : Fragment.t) =
       ; weakdef
       ; always
       ; code
-      ; js_string
-      ; effects
       ; fragment_target
       ; aliases
       ; has_macro
+      ; conditions
+      ; deprecated
       } -> (
-      let ignore_because_of_js_string =
-        match js_string, Config.Flag.use_js_string () with
-        | Some true, false | Some false, true -> true
-        | None, _ | Some true, true | Some false, false -> false
+      let should_ignore =
+        StringMap.exists
+          (fun flag b ->
+            not (Bool.equal b (StringMap.find flag Fragment.allowed_flags ())))
+          conditions
       in
-      let ignore_because_of_effects =
-        match effects, Config.Flag.effects () with
-        | Some true, false | Some false, true -> true
-        | None, _ | Some true, true | Some false, false -> false
-      in
-      if (not version_constraint_ok)
-         || ignore_because_of_js_string
-         || ignore_because_of_effects
+      if (not version_constraint_ok) || should_ignore
       then `Ignored
       else
         match provides with
@@ -530,19 +543,19 @@ let load_fragment ~target_env ~filename (f : Fragment.t) =
               let id = Hashtbl.length provided in
               Primitive.register name kind ka arity;
               StringSet.iter Primitive.register_named_value named_values;
-              Hashtbl.add provided name { id; pi; weakdef; target_env = fragment_target };
+              Hashtbl.add
+                provided
+                name
+                { id; pi; filename; weakdef; target_env = fragment_target };
               Hashtbl.add provided_rev id (name, pi);
-              Hashtbl.add code_pieces id (code, has_macro, requires);
+              Hashtbl.add code_pieces id (code, has_macro, requires, deprecated);
               StringSet.iter (fun alias -> Primitive.alias alias name) aliases;
               `Ok)
 
-let get_provided () =
-  Hashtbl.fold (fun k _ acc -> StringSet.add k acc) provided StringSet.empty
-
 let check_deps () =
-  let provided = get_provided () in
+  let provided = list_all () in
   Hashtbl.iter
-    (fun id (code, _has_macro, requires) ->
+    (fun id (code, _has_macro, requires, _deprecated) ->
       match code with
       | Ok code -> (
           let traverse = new Js_traverse.free in
@@ -588,14 +601,16 @@ let load_files ~target_env l =
   check_deps ()
 
 (* resolve *)
-let rec resolve_dep_name_rev visited path nm =
-  let x =
-    try Hashtbl.find provided nm with Not_found -> error "missing dependency '%s'@." nm
-  in
-  resolve_dep_id_rev visited path x.id
+let rec resolve_dep_name_rev state path nm =
+  match Hashtbl.find provided nm with
+  | x ->
+      if state.include_ x.filename
+      then resolve_dep_id_rev state path x.id
+      else { state with missing = StringSet.add nm state.missing }
+  | exception Not_found -> { state with missing = StringSet.add nm state.missing }
 
-and resolve_dep_id_rev visited path id =
-  if IntSet.mem id visited.ids
+and resolve_dep_id_rev state path id =
+  if IntSet.mem id state.ids
   then (
     if List.memq id ~set:path
     then
@@ -604,55 +619,62 @@ and resolve_dep_id_rev visited path id =
         (String.concat
            ~sep:", "
            (List.map path ~f:(fun id -> fst (Hashtbl.find provided_rev id))));
-    visited)
+    state)
   else
     let path = id :: path in
-    let code, has_macro, req = Hashtbl.find code_pieces id in
-    let visited = { visited with ids = IntSet.add id visited.ids } in
-    let visited =
-      List.fold_left req ~init:visited ~f:(fun visited nm ->
-          resolve_dep_name_rev visited path nm)
+    let code, has_macro, req, deprecated = Hashtbl.find code_pieces id in
+    let state = { state with ids = IntSet.add id state.ids } in
+    let state =
+      List.fold_left req ~init:state ~f:(fun state nm ->
+          resolve_dep_name_rev state path nm)
     in
-    let visited = { visited with codes = (code, has_macro) :: visited.codes } in
-    visited
+    let deprecation =
+      match deprecated with
+      | None -> state.deprecation
+      | Some txt -> (path, txt) :: state.deprecation
+    in
+    let state = { state with codes = (code, has_macro) :: state.codes; deprecation } in
+    state
 
 let proj_always_required { ar_filename; ar_requires; ar_program } =
   { filename = ar_filename; requires = ar_requires; program = unpack ar_program }
 
-let init () =
+let init ?from () =
+  let include_ =
+    match from with
+    | None -> fun _ -> true
+    | Some l -> fun fn -> List.mem fn ~set:l
+  in
   { ids = IntSet.empty
-  ; always_required_codes = List.rev_map !always_included ~f:proj_always_required
+  ; always_required_codes =
+      List.rev
+        (List.filter_map !always_included ~f:(fun x ->
+             if include_ x.ar_filename then Some (proj_always_required x) else None))
+  ; deprecation = []
   ; codes = []
+  ; include_
+  ; missing = StringSet.empty
   }
 
-let resolve_deps ?(linkall = false) visited_rev used =
-  (* link the special files *)
-  let missing, visited_rev =
-    if linkall
-    then
-      (* link all primitives *)
-      let prog, set =
-        Hashtbl.fold
-          (fun nm _ (visited, set) ->
-            resolve_dep_name_rev visited [] nm, StringSet.add nm set)
-          provided
-          (visited_rev, StringSet.empty)
-      in
-      let missing = StringSet.diff used set in
-      missing, prog
-    else
-      (* link used primitives *)
-      StringSet.fold
-        (fun nm (missing, visited) ->
-          if Hashtbl.mem provided nm
-          then missing, resolve_dep_name_rev visited [] nm
-          else StringSet.add nm missing, visited)
-        used
-        (StringSet.empty, visited_rev)
-  in
-  visited_rev, missing
+let do_check_missing state =
+  if not (StringSet.is_empty state.missing)
+  then error "missing dependency '%s'@." (StringSet.choose state.missing)
 
-let link program (state : state) =
+let resolve_deps ?(check_missing = true) state used =
+  (* link the special files *)
+  let missing, state =
+    StringSet.fold
+      (fun nm (missing, visited) ->
+        if Hashtbl.mem provided nm
+        then missing, resolve_dep_name_rev visited [] nm
+        else StringSet.add nm missing, visited)
+      used
+      (StringSet.empty, state)
+  in
+  if check_missing then do_check_missing state;
+  state, missing
+
+let link ?(check_missing = true) program (state : state) =
   let always, always_required =
     List.partition
       ~f:(function
@@ -669,6 +691,30 @@ let link program (state : state) =
         in
         { state with codes = (Ok always.program, false) :: state.codes })
   in
+  if check_missing then do_check_missing state;
+  List.iter state.deprecation ~f:(fun (path, txt) ->
+      match path with
+      | [] -> assert false
+      | [ x ] ->
+          if false
+          then
+            let name = fst (Hashtbl.find provided_rev x) in
+            warn "The runtime primitive [%s] is deprecated. %s\n" name txt
+      | x :: path ->
+          let name = fst (Hashtbl.find provided_rev x) in
+          let path =
+            String.concat
+              ~sep:"\n"
+              (List.map path ~f:(fun id ->
+                   let nm, loc = Hashtbl.find provided_rev id in
+                   Printf.sprintf "-> %s:%s" nm (Parse_info.to_string loc)))
+          in
+          warn
+            "The runtime primitive [%s] is deprecated. %s.  Used by:\n%s\n"
+            name
+            txt
+            path);
+
   let codes =
     List.map state.codes ~f:(fun (x, has_macro) ->
         let c = unpack x in
@@ -691,8 +737,17 @@ let all state =
     state.ids
     []
 
+let missing state = StringSet.elements state.missing
+
 let origin ~name =
   try
     let x = Hashtbl.find provided name in
     x.pi.Parse_info.src
   with Not_found -> None
+
+let deprecated ~name =
+  try
+    let x = Hashtbl.find provided name in
+    let _, _, _, deprecated = Hashtbl.find code_pieces x.id in
+    Option.is_some deprecated
+  with Not_found -> false
