@@ -18,49 +18,34 @@
 
 open! Stdlib
 
-let rec constant_of_const : _ -> Code.constant =
+let rec constant_of_const c : Code.constant =
   let open Lambda in
   let open Asttypes in
-  function
-  | Const_base (Const_int i) -> Int (Int32.of_int_warning_on_overflow i)
-  | Const_base (Const_char c) -> Int (Int32.of_int (Char.code c))
+  match c with
+  | Const_base (Const_int i) -> Int (Targetint.of_int_warning_on_overflow i)
+  | Const_base (Const_char c) -> Int (Targetint.of_int_exn (Char.code c))
   | ((Const_base (Const_string (s, _))) [@if ocaml_version < (4, 11, 0)])
   | ((Const_base (Const_string (s, _, _))) [@if ocaml_version >= (4, 11, 0)]) -> String s
   | Const_base (Const_float s) -> Float (float_of_string s)
-  | Const_base (Const_int32 i) -> Int i
+  | Const_base (Const_int32 i) -> (
+      match Config.target () with
+      | `JavaScript -> Int (Targetint.of_int32_warning_on_overflow i)
+      | `Wasm -> Int32 i)
   | Const_base (Const_int64 i) -> Int64 i
-  | Const_base (Const_nativeint i) -> Int (Int32.of_nativeint_warning_on_overflow i)
+  | Const_base (Const_nativeint i) -> (
+      match Config.target () with
+      | `JavaScript -> Int (Targetint.of_nativeint_warning_on_overflow i)
+      | `Wasm -> NativeInt (Int32.of_nativeint_warning_on_overflow i))
   | Const_immstring s -> String s
   | Const_float_array sl ->
-      let l = List.map ~f:(fun f -> Code.Float (float_of_string f)) sl in
-      Tuple (Obj.double_array_tag, Array.of_list l, Unknown)
+      let l = List.map ~f:(fun f -> float_of_string f) sl in
+      Float_array (Array.of_list l)
   | ((Const_pointer i) [@if ocaml_version < (4, 12, 0)]) ->
-      Int (Int32.of_int_warning_on_overflow i)
+      Int (Targetint.of_int_warning_on_overflow i)
   | Const_block (tag, l) ->
       let l = Array.of_list (List.map l ~f:constant_of_const) in
       Tuple (tag, l, Unknown)
 
-let rec find_loc_in_summary ident' = function
-  | Env.Env_empty -> None
-  | Env.Env_value (_summary, ident, description) when Poly.(ident = ident') ->
-      Some description.Types.val_loc
-  | Env.Env_value (summary, _, _)
-  | Env.Env_type (summary, _, _)
-  | Env.Env_extension (summary, _, _)
-  | Env.Env_module (summary, _, _, _)
-  | Env.Env_modtype (summary, _, _)
-  | Env.Env_class (summary, _, _)
-  | Env.Env_cltype (summary, _, _)
-  | Env.Env_open (summary, _)
-  | Env.Env_functor_arg (summary, _)
-  | Env.Env_constraints (summary, _)
-  | ((Env.Env_copy_types (summary, _)) [@if ocaml_version < (4, 10, 0)])
-  | ((Env.Env_copy_types summary) [@if ocaml_version >= (4, 10, 0)])
-  | Env.Env_persistent (summary, _)
-  | ((Env.Env_value_unbound (summary, _, _)) [@if ocaml_version >= (4, 10, 0)])
-  | ((Env.Env_module_unbound (summary, _, _)) [@if ocaml_version >= (4, 10, 0)]) ->
-      find_loc_in_summary ident' summary
-
 module Symtable = struct
   (* Copied from ocaml/bytecomp/symtable.ml *)
   module Num_tbl (M : Map.S) = struct
@@ -211,6 +196,24 @@ module Symtable = struct
   let current_state () : GlobalMap.t =
     let x : Symtable.global_map = Symtable.current_state () in
     Obj.magic x
+
+  let all_primitives () : string list =
+    let split_primitives p =
+      let len = String.length p in
+      let rec split beg cur =
+        if cur >= len
+        then []
+        else if Char.equal p.[cur] '\000'
+        then String.sub p ~pos:beg ~len:(cur - beg) :: split (cur + 1) (cur + 1)
+        else split beg (cur + 1)
+      in
+      split 0 0
+    in
+    split_primitives (Symtable.data_primitive_names ())
+  [@@if ocaml_version < (5, 2)]
+
+  let all_primitives () : string list = Symtable.data_primitive_names ()
+  [@@if ocaml_version >= (5, 2)]
 end
 
 module Cmo_format = struct
@@ -229,35 +232,24 @@ module Cmo_format = struct
   let requires (t : t) = List.map t.cu_required_compunits ~f:(fun (Compunit u) -> u)
   [@@if ocaml_version >= (5, 2, 0)]
 
+  let provides (t : t) =
+    List.filter_map t.cu_reloc ~f:(fun ((reloc : Cmo_format.reloc_info), _) ->
+        match reloc with
+        | Reloc_setglobal i -> Some (Ident.name i)
+        | Reloc_getglobal _ | Reloc_literal _ | Reloc_primitive _ -> None)
+  [@@if ocaml_version < (5, 2, 0)]
+
+  let provides (t : t) =
+    List.filter_map t.cu_reloc ~f:(fun ((reloc : Cmo_format.reloc_info), _) ->
+        match reloc with
+        | Reloc_setcompunit (Compunit u) -> Some u
+        | Reloc_getcompunit _ | Reloc_getpredef _ | Reloc_literal _ | Reloc_primitive _ ->
+            None)
+  [@@if ocaml_version >= (5, 2, 0)]
+
   let primitives (t : t) = t.cu_primitives
 
   let imports (t : t) = t.cu_imports
 
   let force_link (t : t) = t.cu_force_link
 end
-
-module Ident = struct
-  [@@@ocaml.warning "-unused-field"]
-
-  (* Copied from ocaml/typing/ident.ml *)
-  type 'a tbl' =
-    | Empty
-    | Node of 'a tbl' * 'a data * 'a tbl' * int
-
-  and 'a data =
-    { ident : Ident.t
-    ; data : 'a
-    ; previous : 'a data option
-    }
-
-  type 'a tbl = 'a Ident.tbl
-
-  let rec table_contents_rec t rem =
-    match t with
-    | Empty -> rem
-    | Node (l, v, r, _) ->
-        table_contents_rec l ((v.data, v.ident) :: table_contents_rec r rem)
-
-  let table_contents (t : 'a tbl) =
-    table_contents_rec (Obj.magic (t : 'a tbl) : 'a tbl') []
-end
