@@ -307,32 +307,25 @@ module Int32 = struct
 
   external equal : int32 -> int32 -> bool = "%equal"
 
-  let warn_overflow ~to_dec ~to_hex i i32 =
+  let warn_overflow name ~to_dec ~to_hex i i32 =
     warn
-      "Warning: integer overflow: integer 0x%s (%s) truncated to 0x%lx (%ld); the \
-       generated code might be incorrect.@."
+      "Warning: integer overflow: %s 0x%s (%s) truncated to 0x%lx (%ld); the generated \
+       code might be incorrect.@."
+      name
       (to_hex i)
       (to_dec i)
       i32
       i32
 
-  let convert_warning_on_overflow ~to_int32 ~of_int32 ~equal ~to_dec ~to_hex x =
+  let convert_warning_on_overflow name ~to_int32 ~of_int32 ~equal ~to_dec ~to_hex x =
     let i32 = to_int32 x in
     let x' = of_int32 i32 in
-    if not (equal x' x) then warn_overflow ~to_dec ~to_hex x i32;
+    if not (equal x' x) then warn_overflow name ~to_dec ~to_hex x i32;
     i32
 
-  let of_int_warning_on_overflow i =
-    convert_warning_on_overflow
-      ~to_int32:Int32.of_int
-      ~of_int32:Int32.to_int
-      ~equal:Int_replace_polymorphic_compare.( = )
-      ~to_dec:(Printf.sprintf "%d")
-      ~to_hex:(Printf.sprintf "%x")
-      i
-
   let of_nativeint_warning_on_overflow n =
     convert_warning_on_overflow
+      "native integer"
       ~to_int32:Nativeint.to_int32
       ~of_int32:Nativeint.of_int32
       ~equal:Nativeint.equal
@@ -395,13 +388,17 @@ end
 module Int64 = struct
   include Int64
 
-  let equal (a : int64) (b : int64) = Poly.( = ) a b
+  let equal (a : int64) (b : int64) = Poly.(a = b)
 end
 
 module Float = struct
   type t = float
 
-  let equal (a : float) (b : float) =
+  let equal (_ : float) (_ : float) = `Use_ieee_equal_or_bitwise_equal
+
+  let ieee_equal (a : float) (b : float) = Poly.equal a b
+
+  let bitwise_equal (a : float) (b : float) =
     Int64.equal (Int64.bits_of_float a) (Int64.bits_of_float b)
 
   (* Re-defined here to stay compatible with OCaml 4.02 *)
@@ -571,6 +568,20 @@ module Bytes = struct
   include BytesLabels
 
   let sub_string b ~pos:ofs ~len = unsafe_to_string (Bytes.sub b ofs len)
+
+  let fold_left ~f ~init b =
+    let r = ref init in
+    for i = 0 to length b - 1 do
+      r := f !r (unsafe_get b i)
+    done;
+    !r
+
+  let fold_right ~f b ~init =
+    let r = ref init in
+    for i = length b - 1 downto 0 do
+      r := f (unsafe_get b i) !r
+    done;
+    !r
 end
 
 module String = struct
@@ -726,6 +737,12 @@ module String = struct
       Some (sub line ~pos:0 ~len:pos, sub line ~pos:(pos + 1) ~len:(length line - pos - 1))
     with Not_found -> None
 
+  let rsplit2 line ~on:delim =
+    try
+      let pos = rindex line delim in
+      Some (sub line ~pos:0 ~len:pos, sub line ~pos:(pos + 1) ~len:(length line - pos - 1))
+    with Not_found -> None
+
   let capitalize_ascii s = apply1 Char.uppercase_ascii s
 
   let uncapitalize_ascii s = apply1 Char.lowercase_ascii s
@@ -980,6 +997,20 @@ module String = struct
         | _ -> false
     in
     loop (length b - 1) b 0
+
+  let fold_left ~f ~init s =
+    let r = ref init in
+    for i = 0 to length s - 1 do
+      r := f !r (unsafe_get s i)
+    done;
+    !r
+
+  let fold_right ~f s ~init =
+    let r = ref init in
+    for i = length s - 1 downto 0 do
+      r := f (unsafe_get s i) !r
+    done;
+    !r
 end
 
 module Utf8_string : sig
@@ -1113,6 +1144,17 @@ end
 module Array = struct
   include ArrayLabels
 
+  let find_opt ~f:p a =
+    let n = length a in
+    let rec loop i =
+      if i = n
+      then None
+      else
+        let x = unsafe_get a i in
+        if p x then Some x else loop (succ i)
+    in
+    loop 0
+
   let fold_right_i a ~f ~init:x =
     let r = ref x in
     for i = Array.length a - 1 downto 0 do
@@ -1149,13 +1191,16 @@ module Filename = struct
     in
     try
       let ch = open_out_bin f_tmp in
-      (try f ch
-       with e ->
-         close_out ch;
-         raise e);
+      let res =
+        try f ch
+        with e ->
+          close_out ch;
+          raise e
+      in
       close_out ch;
       (try Sys.remove file with Sys_error _ -> ());
-      Sys.rename f_tmp file
+      Sys.rename f_tmp file;
+      res
     with exc ->
       Sys.remove f_tmp;
       raise exc
