@@ -29,7 +29,7 @@ let expr s e =
   | Apply { f; args; exact } ->
       Apply { f = s f; args = List.map args ~f:(fun x -> s x); exact }
   | Block (n, a, k, mut) -> Block (n, Array.map a ~f:(fun x -> s x), k, mut)
-  | Field (x, n) -> Field (s x, n)
+  | Field (x, n, typ) -> Field (s x, n, typ)
   | Closure (l, pc) -> Closure (l, subst_cont s pc)
   | Special _ -> e
   | Prim (p, l) ->
@@ -44,25 +44,23 @@ let instr s i =
   match i with
   | Let (x, e) -> Let (x, expr s e)
   | Assign (x, y) -> Assign (x, s y) (* x is handled like a parameter *)
-  | Set_field (x, n, y) -> Set_field (s x, n, s y)
+  | Set_field (x, n, typ, y) -> Set_field (s x, n, typ, s y)
   | Offset_ref (x, n) -> Offset_ref (s x, n)
   | Array_set (x, y, z) -> Array_set (s x, s y, s z)
+  | Event _ -> i
 
-let instrs s l = List.map l ~f:(fun (i, loc) -> instr s i, loc)
+let instrs s l = List.map l ~f:(fun i -> instr s i)
 
-let last s (l, loc) =
-  let l =
-    match l with
-    | Stop -> l
-    | Branch cont -> Branch (subst_cont s cont)
-    | Pushtrap (cont1, x, cont2) -> Pushtrap (subst_cont s cont1, x, subst_cont s cont2)
-    | Return x -> Return (s x)
-    | Raise (x, k) -> Raise (s x, k)
-    | Cond (x, cont1, cont2) -> Cond (s x, subst_cont s cont1, subst_cont s cont2)
-    | Switch (x, a1) -> Switch (s x, Array.map a1 ~f:(fun cont -> subst_cont s cont))
-    | Poptrap cont -> Poptrap (subst_cont s cont)
-  in
-  l, loc
+let last s l =
+  match l with
+  | Stop -> l
+  | Branch cont -> Branch (subst_cont s cont)
+  | Pushtrap (cont1, x, cont2) -> Pushtrap (subst_cont s cont1, x, subst_cont s cont2)
+  | Return x -> Return (s x)
+  | Raise (x, k) -> Raise (s x, k)
+  | Cond (x, cont1, cont2) -> Cond (s x, subst_cont s cont1, subst_cont s cont2)
+  | Switch (x, a1) -> Switch (s x, Array.map a1 ~f:(fun cont -> subst_cont s cont))
+  | Poptrap cont -> Poptrap (subst_cont s cont)
 
 let block s block =
   { params = block.params; body = instrs s block.body; branch = last s block.branch }
@@ -80,10 +78,7 @@ let rec cont' s pc blocks visited =
     let b = block s b in
     let blocks = Addr.Map.add pc b blocks in
     let blocks, visited =
-      List.fold_left
-        b.body
-        ~init:(blocks, visited)
-        ~f:(fun (blocks, visited) (instr, _) ->
+      List.fold_left b.body ~init:(blocks, visited) ~f:(fun (blocks, visited) instr ->
           match instr with
           | Let (_, Closure (_, (pc, _))) -> cont' s pc blocks visited
           | _ -> blocks, visited)
