@@ -35,50 +35,35 @@ let rec remove_last l =
 let rec tail_call x f l =
   match l with
   | [] -> None
-  | [ (Let (y, Apply { f = g; args; _ }), loc) ]
-    when Var.compare x y = 0 && Var.compare f g = 0 -> Some (args, loc)
+  | [ Let (y, Apply { f = g; args; _ }) ] when Var.compare x y = 0 && Var.compare f g = 0
+    -> Some args
   | _ :: rem -> tail_call x f rem
 
 let rewrite_block (f, f_params, f_pc, args) pc blocks =
   let block = Addr.Map.find pc blocks in
   match block.branch with
-  | Return x, _loc -> (
+  | Return x -> (
       match tail_call x f block.body with
-      | Some (f_args, loc) when List.length f_params = List.length f_args ->
+      | Some f_args when List.length f_params = List.length f_args ->
           let m = Subst.build_mapping f_params f_args in
           List.iter2 f_params f_args ~f:(fun p a -> Code.Var.propagate_name p a);
           Addr.Map.add
             pc
             { params = block.params
             ; body = remove_last block.body
-            ; branch = Branch (f_pc, List.map args ~f:(fun x -> Var.Map.find x m)), loc
+            ; branch = Branch (f_pc, List.map args ~f:(fun x -> Var.Map.find x m))
             }
             blocks
       | _ -> blocks)
   | _ -> blocks
 
-(* Skip try body *)
-let fold_children blocks pc f accu =
-  let block = Addr.Map.find pc blocks in
-  match fst block.branch with
-  | Return _ | Raise _ | Stop -> accu
-  | Branch (pc', _) | Poptrap (pc', _) -> f pc' accu
-  | Pushtrap (_, _, (pc1, _), pcs) -> f pc1 (Addr.Set.fold f pcs accu)
-  | Cond (_, (pc1, _), (pc2, _)) ->
-      let accu = f pc1 accu in
-      let accu = f pc2 accu in
-      accu
-  | Switch (_, a1) ->
-      let accu = Array.fold_right a1 ~init:accu ~f:(fun (pc, _) accu -> f pc accu) in
-      accu
-
 let rec traverse f pc visited blocks =
   if not (Addr.Set.mem pc visited)
   then
     let visited = Addr.Set.add pc visited in
     let blocks = rewrite_block f pc blocks in
     let visited, blocks =
-      fold_children
+      Code.fold_children_skip_try_body
         blocks
         pc
         (fun pc (visited, blocks) ->
