@@ -34,7 +34,6 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
   print_program code;
   [%expect
     {|
-
     (function(globalThis){
        "use strict";
        var
@@ -52,7 +51,7 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
                     >= 0
                     ? f.l
                     : f.l = f.length)
-                  == 2
+                  === 2
                   ? f(a0, a1)
                   : runtime.caml_call_gen(f, [a0, a1])
                 : runtime.caml_trampoline_return(f, [a0, a1]);
@@ -65,7 +64,7 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
        return caml_callback
                (function(cont){
                  var
-                  undef = undefined,
+                  dummy = 0,
                   global_data = runtime.caml_get_global_data(),
                   Stdlib_Printf = global_data.Stdlib__Printf,
                   _a_ =
@@ -75,18 +74,17 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
                  function g(param, cont){
                   return caml_cps_call2(Stdlib_Printf[2], _a_, cont);
                  }
-                 caml_callback(g, [undef]);
+                 caml_callback(g, [dummy]);
                  function _b_(i){
                   return caml_cps_exact_call2
                           (g,
-                           undef,
+                           dummy,
                            function(_c_){
                             var _d_ = i + 1 | 0;
                             if(5 !== i) return caml_cps_exact_call1(_b_, _d_);
-                            caml_callback(g, [undef]);
+                            caml_callback(g, [dummy]);
                             var Test = [0];
                             runtime.caml_register_global(2, Test, "Test");
-                            return;
                            });
                  }
                  return _b_(1);
@@ -94,4 +92,5 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
                 []);
       }
       (globalThis));
-    //end |}]
+    //end
+    |}]
