@@ -211,10 +211,10 @@ let ()  = M.myfun M.x
           var
            n1 = n >> 1,
            n2 = n - n1 | 0,
-           match$0 = rev_sort(n1, l),
+           match$0 = rev_sort$0(n1, l),
            l2$0 = match$0[2],
            s1 = match$0[1],
-           match$1 = rev_sort(n2, l2$0),
+           match$1 = rev_sort$0(n2, l2$0),
            tl$0 = match$1[2],
            s2 = match$1[1],
            l1 = s1,
@@ -225,10 +225,14 @@ let ()  = M.myfun M.x
             if(l2){
              var t2 = l2[2], h2 = l2[1], t1 = l1[2], h1 = l1[1];
              if(0 < caml_int_compare(h1, h2)){
-              var accu$0 = [0, h1, accu], l1 = t1, accu = accu$0;
+              var accu$0 = [0, h1, accu];
+              l1 = t1;
+              accu = accu$0;
               continue;
              }
-             var accu$1 = [0, h2, accu], l2 = t2, accu = accu$1;
+             var accu$1 = [0, h2, accu];
+             l2 = t2;
+             accu = accu$1;
              continue;
             }
             var _c_ = rev_append(l1, accu);
@@ -289,10 +293,10 @@ let ()  = M.myfun M.x
           var
            n1 = n >> 1,
            n2 = n - n1 | 0,
-           match$0 = sort(n1, l),
+           match$0 = sort$0(n1, l),
            l2$0 = match$0[2],
            s1 = match$0[1],
-           match$1 = sort(n2, l2$0),
+           match$1 = sort$0(n2, l2$0),
            tl$0 = match$1[2],
            s2 = match$1[1],
            l1 = s1,
@@ -303,10 +307,14 @@ let ()  = M.myfun M.x
             if(l2){
              var t2 = l2[2], h2 = l2[1], t1 = l1[2], h1 = l1[1];
              if(0 < caml_int_compare(h1, h2)){
-              var accu$0 = [0, h2, accu], l2 = t2, accu = accu$0;
+              var accu$0 = [0, h2, accu];
+              l2 = t2;
+              accu = accu$0;
               continue;
              }
-             var accu$1 = [0, h1, accu], l1 = t1, accu = accu$1;
+             var accu$1 = [0, h1, accu];
+             l1 = t1;
+             accu = accu$1;
              continue;
             }
             var _a_ = rev_append(l1, accu);
@@ -315,15 +323,17 @@ let ()  = M.myfun M.x
             var _a_ = rev_append(l2, accu);
            return [0, _a_, tl$0];
           }
-         },
-       len = 0,
-       param = l;
+         };
+      let sort$0 = sort, rev_sort$0 = rev_sort;
+      var len = 0, param = l;
       for(;;){
        if(! param) break;
-       var l$0 = param[2], len$0 = len + 1 | 0, len = len$0, param = l$0;
+       var l$0 = param[2], len$0 = len + 1 | 0;
+       len = len$0;
+       param = l$0;
       }
       if(2 <= len) sort(len, l);
-      var x$0 = next;
+      x$0 = next;
      }
     }
     //end |}]
@@ -368,30 +378,31 @@ let ()  = M.run ()
       var
        odd =
          function(n){
-          if(2 < n >>> 0) return 1 - (1 - even(n - 1 | 0));
+          if(2 < n >>> 0) return 1 - (1 - even$0(n - 1 | 0));
           switch(n){
             case 0:
              return 0;
             case 1:
-             return 1 - (1 - even(0));
-            default: return 1 - (1 - even(1));
+             return 1 - (1 - even$0(0));
+            default: return 1 - (1 - even$0(1));
           }
          },
        even =
          function(n){
-          if(2 < n >>> 0) return 1 - (1 - odd(n - 1 | 0));
+          if(2 < n >>> 0) return 1 - (1 - odd$0(n - 1 | 0));
           switch(n){
             case 0:
              return 1;
             case 1:
-             return 1 - (1 - odd(0));
-            default: return 1 - (1 - odd(1));
+             return 1 - (1 - odd$0(0));
+            default: return 1 - (1 - odd$0(1));
           }
          };
+      let odd$0 = odd, even$0 = even;
       if(even(i)) caml_call1(Stdlib[42], cst);
       var _a_ = i + 1 | 0;
       if(4 === i) return 0;
-      var i = _a_;
+      i = _a_;
      }
     }
     //end |}]
@@ -461,50 +472,48 @@ let ()  = M.run ()
     function run(param){
      var i = 0;
      for(;;){
+      let i$0 = i;
       var
-       closures =
-         function(i){
-          function odd(n){
-           if(2 < n >>> 0) return 1 - (1 - even(n - 1 | 0));
-           switch(n){
-             case 0:
-              var
-               f = function(param){return caml_call2(Stdlib_Printf[2], _b_, i);};
-              delayed[1] = [0, f, delayed[1]];
-              f(0);
-              return 0;
-             case 1:
-              return 1 - (1 - even(0));
-             default: return 1 - (1 - even(1));
-           }
+       odd =
+         function(n){
+          if(2 < n >>> 0) return 1 - (1 - even$0(n - 1 | 0));
+          switch(n){
+            case 0:
+             var
+              f = function(param){return caml_call2(Stdlib_Printf[2], _a_, i$0);};
+             delayed[1] = [0, f, delayed[1]];
+             f(0);
+             return 0;
+            case 1:
+             return 1 - (1 - even$0(0));
+            default: return 1 - (1 - even$0(1));
           }
-          function even(n){
-           if(2 < n >>> 0) return 1 - (1 - odd(n - 1 | 0));
-           switch(n){
-             case 0:
-              var
-               f = function(param){return caml_call2(Stdlib_Printf[2], _c_, i);};
-              delayed[1] = [0, f, delayed[1]];
-              f(0);
-              return 1;
-             case 1:
-              return 1 - (1 - odd(0));
-             default: return 1 - (1 - odd(1));
-           }
-          }
-          var block = [0, odd, even];
-          return block;
          },
-       closures$0 = closures(i),
-       even = closures$0[2];
+       even =
+         function(n){
+          if(2 < n >>> 0) return 1 - (1 - odd$0(n - 1 | 0));
+          switch(n){
+            case 0:
+             var
+              f = function(param){return caml_call2(Stdlib_Printf[2], _b_, i$0);};
+             delayed[1] = [0, f, delayed[1]];
+             f(0);
+             return 1;
+            case 1:
+             return 1 - (1 - odd$0(0));
+            default: return 1 - (1 - odd$0(1));
+          }
+         };
+      let odd$0 = odd, even$0 = even;
       if(even(i)) caml_call1(Stdlib[42], cst);
-      var _e_ = i + 1 | 0;
-      if(4 === i){
-       var _d_ = caml_call1(list_rev, delayed[1]);
-       return caml_call2(list_iter, function(f){return caml_call1(f, 0);}, _d_);
-      }
-      var i = _e_;
+      var _c_ = i + 1 | 0;
+      if(4 === i) break;
+      i = _c_;
      }
+     return caml_call2
+             (list_iter,
+              function(f){return caml_call1(f, 0);},
+              caml_call1(list_rev, delayed[1]));
     }
     //end |}]
 
@@ -577,65 +586,64 @@ let ()  = M.run ()
     function run(param){
      var i = 0;
      for(;;){
+      let i$0 = i;
       var
-       closures =
-         function(i){
-          function odd(n){
-           if(2 < n >>> 0)
-            return [0, 748545554, function(param){return even(n - 1 | 0);}];
-           switch(n){
-             case 0:
-              return [0,
-                      748545554,
-                      function(param){
-                       function f(param){
-                        return caml_call2(Stdlib_Printf[2], _b_, i);
-                       }
-                       delayed[1] = [0, f, delayed[1]];
-                       f(0);
-                       return _c_;
-                      }];
-             case 1:
-              return [0, 748545554, function(param){return even(0);}];
-             default: return [0, 748545554, function(param){return even(1);}];
-           }
+       odd =
+         function(n){
+          if(2 < n >>> 0)
+           return [0, 748545554, function(param){return even$0(n - 1 | 0);}];
+          switch(n){
+            case 0:
+             return [0,
+                     748545554,
+                     function(param){
+                      function f(param){
+                       return caml_call2(Stdlib_Printf[2], _a_, i$0);
+                      }
+                      delayed[1] = [0, f, delayed[1]];
+                      f(0);
+                      return _b_;
+                     }];
+            case 1:
+             return [0, 748545554, function(param){return even$0(0);}];
+            default: return [0, 748545554, function(param){return even$0(1);}];
           }
-          function even(n){
-           if(2 < n >>> 0)
-            return [0, 748545554, function(param){return odd(n - 1 | 0);}];
-           switch(n){
-             case 0:
-              return [0,
-                      748545554,
-                      function(param){
-                       function f(param){
-                        return caml_call2(Stdlib_Printf[2], _d_, i);
-                       }
-                       delayed[1] = [0, f, delayed[1]];
-                       f(0);
-                       return _e_;
-                      }];
-             case 1:
-              return [0, 748545554, function(param){return odd(0);}];
-             default: return [0, 748545554, function(param){return odd(1);}];
-           }
-          }
-          var block = [0, odd, even];
-          return block;
          },
-       closures$0 = closures(i),
-       even = closures$0[2],
-       param$0 = even(i);
+       even =
+         function(n){
+          if(2 < n >>> 0)
+           return [0, 748545554, function(param){return odd$0(n - 1 | 0);}];
+          switch(n){
+            case 0:
+             return [0,
+                     748545554,
+                     function(param){
+                      function f(param){
+                       return caml_call2(Stdlib_Printf[2], _c_, i$0);
+                      }
+                      delayed[1] = [0, f, delayed[1]];
+                      f(0);
+                      return _d_;
+                     }];
+            case 1:
+             return [0, 748545554, function(param){return odd$0(0);}];
+            default: return [0, 748545554, function(param){return odd$0(1);}];
+          }
+         };
+      let odd$0 = odd, even$0 = even;
+      var param$0 = even(i);
       for(;;){
        if(759635106 <= param$0[1]) break;
-       var f = param$0[2], param$0 = f(0);
+       var f = param$0[2];
+       param$0 = f(0);
       }
-      var _g_ = i + 1 | 0;
-      if(4 === i){
-       var _f_ = caml_call1(list_rev, delayed[1]);
-       return caml_call2(list_iter, function(f){return caml_call1(f, 0);}, _f_);
-      }
-      var i = _g_;
+      var _e_ = i + 1 | 0;
+      if(4 === i) break;
+      i = _e_;
      }
+     return caml_call2
+             (list_iter,
+              function(f){return caml_call1(f, 0);},
+              caml_call1(list_rev, delayed[1]));
     }
     //end |}]
