@@ -25,19 +25,21 @@ let%expect_test _ =
   Util.compile_and_run ~skip_modern:true prog;
   [%expect
     {|
-Warning: integer overflow: integer 0xffffffff (4294967295) truncated to 0xffffffff (-1); the generated code might be incorrect.
-ffffffff |}];
+    Warning: integer overflow: native integer 0xffffffff (4294967295) truncated to 0xffffffff (-1); the generated code might be incorrect.
+    ffffffff
+    |}];
   ()
 
 let%expect_test _ =
   Util.print_fun_decl (Util.compile_and_parse prog) None;
   [%expect
     {|
-    Warning: integer overflow: integer 0xffffffff (4294967295) truncated to 0xffffffff (-1); the generated code might be incorrect.
+    Warning: integer overflow: native integer 0xffffffff (4294967295) truncated to 0xffffffff (-1); the generated code might be incorrect.
     function caml_call2(f, a0, a1){
-     return (f.l >= 0 ? f.l : f.l = f.length) == 2
+     return (f.l >= 0 ? f.l : f.l = f.length) === 2
              ? f(a0, a1)
              : runtime.caml_call_gen(f, [a0, a1]);
     }
-    //end |}];
+    //end
+    |}];
   ()
