@@ -1,8 +1,8 @@
+external unregister : string -> unit = "caml_unregister_named_value"
+
 let () =
   match Array.to_list Sys.argv with
-  | _ :: "unregister" :: _ ->
-      let null = Array.unsafe_get [| 1 |] 1 in
-      Callback.register "Printexc.handle_uncaught_exception" null
+  | _ :: "unregister" :: _ -> unregister "Printexc.handle_uncaught_exception"
   | _ -> ()
 
 exception D of int * string * Int64.t
