@@ -65,64 +65,64 @@ let on_cube c i j k f =
   let x = float (i - k + n - 1) *. w in
   let y = (float (n - 1 - j) *. h) +. (float (i + k) *. h /. 2.) in
   c##save;
-  c##translate x y;
+  c##translate (Js.float x) (Js.float y);
   f c;
   c##restore
 
 let draw_top c =
   c##.fillStyle := top;
   c##beginPath;
-  c##moveTo w 0.;
-  c##lineTo (2. *. w) (h /. 2.);
-  c##lineTo w h;
-  c##lineTo 0. (h /. 2.);
+  c##moveTo (Js.float w) (Js.float 0.);
+  c##lineTo (Js.float (2. *. w)) (Js.float (h /. 2.));
+  c##lineTo (Js.float w) (Js.float h);
+  c##lineTo (Js.float 0.) (Js.float (h /. 2.));
   c##fill
 
 let top_edges c =
   c##beginPath;
-  c##moveTo 0. (h /. 2.);
-  c##lineTo w 0.;
-  c##lineTo (2. *. w) (h /. 2.);
+  c##moveTo (Js.float 0.) (Js.float (h /. 2.));
+  c##lineTo (Js.float w) (Js.float 0.);
+  c##lineTo (Js.float (2. *. w)) (Js.float (h /. 2.));
   c##stroke
 
 let draw_right c =
   c##.fillStyle := right;
   c##beginPath;
-  c##moveTo w h;
-  c##lineTo w (2. *. h);
-  c##lineTo (2. *. w) (1.5 *. h);
-  c##lineTo (2. *. w) (h /. 2.);
+  c##moveTo (Js.float w) (Js.float h);
+  c##lineTo (Js.float w) (Js.float (2. *. h));
+  c##lineTo (Js.float (2. *. w)) (Js.float (1.5 *. h));
+  c##lineTo (Js.float (2. *. w)) (Js.float (h /. 2.));
   c##fill
 
 let right_edges c =
   c##beginPath;
-  c##moveTo w (2. *. h);
-  c##lineTo w h;
-  c##lineTo (2. *. w) (h /. 2.);
+  c##moveTo (Js.float w) (Js.float (2. *. h));
+  c##lineTo (Js.float w) (Js.float h);
+  c##lineTo (Js.float (2. *. w)) (Js.float (h /. 2.));
   c##stroke
 
 let draw_left c =
   c##.fillStyle := left;
   c##beginPath;
-  c##moveTo w h;
-  c##lineTo w (2. *. h);
-  c##lineTo 0. (1.5 *. h);
-  c##lineTo 0. (h /. 2.);
+  c##moveTo (Js.float w) (Js.float h);
+  c##lineTo (Js.float w) (Js.float (2. *. h));
+  c##lineTo (Js.float 0.) (Js.float (1.5 *. h));
+  c##lineTo (Js.float 0.) (Js.float (h /. 2.));
   c##fill
 
 let left_edges c =
   c##beginPath;
-  c##moveTo w h;
-  c##lineTo 0. (h /. 2.);
-  c##lineTo 0. (1.5 *. h);
+  c##moveTo (Js.float w) (Js.float h);
+  c##lineTo (Js.float 0.) (Js.float (h /. 2.));
+  c##lineTo (Js.float 0.) (Js.float (1.5 *. h));
   c##stroke
 
 let remaining_edges c =
   c##beginPath;
-  c##moveTo 0. (float n *. 1.5 *. h);
-  c##lineTo (float n *. w) (float n *. 2. *. h);
-  c##lineTo (float n *. 2. *. w) (float n *. 1.5 *. h);
-  c##lineTo (float n *. 2. *. w) (float n *. 0.5 *. h);
+  c##moveTo (Js.float 0.) (Js.float (float n *. 1.5 *. h));
+  c##lineTo (Js.float (float n *. w)) (Js.float (float n *. 2. *. h));
+  c##lineTo (Js.float (float n *. 2. *. w)) (Js.float (float n *. 1.5 *. h));
+  c##lineTo (Js.float (float n *. 2. *. w)) (Js.float (float n *. 0.5 *. h));
   c##stroke
 
 let tile c a (top, right, left) =
@@ -163,15 +163,31 @@ let create_canvas () =
 
 let redraw ctx canvas a =
   let c = canvas##getContext Html._2d_ in
-  c##setTransform 1. 0. 0. 1. 0. 0.;
-  c##clearRect 0. 0. (float canvas##.width) (float canvas##.height);
-  c##setTransform 1. 0. 0. 1. 0.5 0.5;
+  c##setTransform
+    (Js.float 1.)
+    (Js.float 0.)
+    (Js.float 0.)
+    (Js.float 1.)
+    (Js.float 0.)
+    (Js.float 0.);
+  c##clearRect
+    (Js.float 0.)
+    (Js.float 0.)
+    (Js.float (float canvas##.width))
+    (Js.float (float canvas##.height));
+  c##setTransform
+    (Js.float 1.)
+    (Js.float 0.)
+    (Js.float 0.)
+    (Js.float 1.)
+    (Js.float 0.5)
+    (Js.float 0.5);
   c##.globalCompositeOperation := Js.string "lighter";
   tile c a (draw_top, draw_right, draw_left);
   c##.globalCompositeOperation := Js.string "source-over";
   tile c a (top_edges, right_edges, left_edges);
   remaining_edges c;
-  ctx##drawImage_fromCanvas canvas 0. 0.
+  ctx##drawImage_fromCanvas canvas (Js.float 0.) (Js.float 0.)
 
 let ( >>= ) = Lwt.bind
 
