@@ -104,9 +104,9 @@ let outside_color = Js.string (*"#0c1a0d"*) "#070718"
 let option var = Js.Optdef.get var (fun () -> Js.Unsafe.coerce (new%js Js.array_empty))
 
 class type style = object
-  method border : float Js.optdef Js.readonly_prop
+  method border : Js.number_t Js.optdef Js.readonly_prop
 
-  method padding : float Js.optdef Js.readonly_prop
+  method padding : Js.number_t Js.optdef Js.readonly_prop
 
   method backgroundColor : Js.js_string Js.t Js.optdef Js.readonly_prop
 
@@ -553,7 +553,10 @@ let local_messages msgs : messages Js.t = option (Js.Unsafe.get msgs !language)
 (******)
 
 let screen_transform canvas =
-  let offset = opt_style style##.border 0.5 +. opt_style style##.padding 0. in
+  let offset =
+    Js.to_float (opt_style style##.border (Js.float 0.5))
+    +. Js.to_float (opt_style style##.padding (Js.float 0.))
+  in
   let w = canvas##.width in
   let h = canvas##.height in
   (*
@@ -580,47 +583,21 @@ let from_screen canvas x y =
 
 let pi = 4. *. atan 1.
 
-let ellipse_arc c cx cy rx ry start fin clock_wise =
-  c##save;
-  c##translate (Js.float cx) (Js.float cy);
-  c##scale (Js.float rx) (Js.float ry);
-  c##arc
-    (Js.float 0.)
-    (Js.float 0.)
-    (Js.float 1.)
-    (Js.float start)
-    (Js.float fin)
-    clock_wise;
-  c##restore
-
 let arc c (rx, ry, dx, dy) z0 z1 z2 =
   let rd = norm (sub z1 z0) in
   let start = atan2 (z1.y -. z0.y) (z1.x -. z0.x) in
   let fin = atan2 (z2.y -. z0.y) (z2.x -. z0.x) in
   c##beginPath;
   let alpha = mod_float (fin -. start +. (2. *. pi)) (2. *. pi) in
-  (*
-Firebug.console##log_4(start, fin, alpha, (alpha > pi));
-*)
-  if rx = ry
-  then
-    c##arc
-      (Js.float ((z0.x *. rx) +. dx))
-      (Js.float ((z0.y *. rx) +. dy))
-      (Js.float (rd *. rx))
-      (Js.float start)
-      (Js.float fin)
-      (Js.bool (alpha > pi))
-  else
-    ellipse_arc
-      c
-      ((z0.x *. rx) +. dx)
-      ((z0.y *. ry) +. dy)
-      (rd *. rx)
-      (rd *. ry)
-      start
-      fin
-      (Js.bool (alpha > pi));
+  c##ellipse
+    (Js.float ((z0.x *. rx) +. dx))
+    (Js.float ((z0.y *. ry) +. dy))
+    (Js.float (rd *. rx))
+    (Js.float (rd *. ry))
+    (Js.float 0.)
+    (Js.float start)
+    (Js.float fin)
+    (Js.bool (alpha > pi));
   c##stroke
 
 let line c (rx, ry, dx, dy) z1 z2 =
@@ -666,9 +643,17 @@ let draw canvas vertices edges nodes boxes =
     (Js.float 0.)
     (Js.float (float canvas##.width))
     (Js.float (float canvas##.height));
-  let padding = opt_style style##.padding 0. in
+  let padding = Js.to_float (opt_style style##.padding (Js.float 0.)) in
   c##beginPath;
-  ellipse_arc c dx dy (rx +. padding) (ry +. padding) 0. 7. Js._false;
+  c##ellipse
+    (Js.float dx)
+    (Js.float dy)
+    (Js.float (rx +. padding))
+    (Js.float (ry +. padding))
+    (Js.float 0.)
+    (Js.float 0.)
+    (Js.float 7.)
+    Js._false;
   Js.Optdef.iter style##.backgroundColor (fun color ->
       c##.fillStyle := color;
       c##fill);
@@ -1133,12 +1118,12 @@ let close_button over =
     c##.shadowBlur := Js.float offset;
     c##.shadowColor := color);
   c##beginPath;
-  let a = Js.float (offset +. (lw /. sqrt 2.)) in
-  let b = Js.float (float size -. offset -. (lw /. sqrt 2.)) in
-  c##moveTo a a;
-  c##lineTo b b;
-  c##moveTo a b;
-  c##lineTo b a;
+  let a = offset +. (lw /. sqrt 2.) in
+  let b = float size -. offset -. (lw /. sqrt 2.) in
+  c##moveTo (Js.float a) (Js.float a);
+  c##lineTo (Js.float b) (Js.float b);
+  c##moveTo (Js.float a) (Js.float b);
+  c##lineTo (Js.float b) (Js.float a);
   c##stroke;
   c##restore;
   canvas##.className := Js.string (if over then "on" else "off");
