@@ -27,7 +27,10 @@ let onIE = Js.to_bool (caml_js_on_ie ())
 
 external html_escape : js_string t -> js_string t = "caml_js_html_escape"
 
-external decode_html_entities : js_string t -> js_string t = "caml_js_html_entities"
+external html_entities : js_string t -> js_string t opt = "caml_js_html_entities"
+
+let decode_html_entities s =
+  Js.Opt.get (html_entities s) (fun () -> failwith ("Invalid entity " ^ Js.to_string s))
 
 class type cssStyleDeclaration = object
   method setProperty :
@@ -449,6 +452,14 @@ and clipboardEvent = object
   method clipboardData : dataTransfer t readonly_prop
 end
 
+and toggleEvent = object
+  inherit event
+
+  method newState : js_string t readonly_prop
+
+  method oldState : js_string t readonly_prop
+end
+
 and dataTransfer = object
   method dropEffect : js_string t prop
 
@@ -770,8 +781,12 @@ let invoke_handler = Dom.invoke_handler
 module Event = struct
   type 'a typ = 'a Dom.Event.typ
 
+  let cancel = Dom.Event.make "cancel"
+
   let click = Dom.Event.make "click"
 
+  let close = Dom.Event.make "close"
+
   let copy = Dom.Event.make "copy"
 
   let cut = Dom.Event.make "cut"
@@ -952,6 +967,8 @@ module Event = struct
 
   let waiting = Dom.Event.make "waiting"
 
+  let toggle = Dom.Event.make "toggle"
+
   let make = Dom.Event.make
 end
 
@@ -1289,6 +1306,26 @@ class type dListElement = element
 
 class type liElement = element
 
+class type dialogElement = object
+  inherit element
+
+  method close : unit meth
+
+  method close_returnValue : js_string t -> unit meth
+
+  method open_ : bool t prop
+
+  method returnValue : js_string t prop
+
+  method show : unit meth
+
+  method showModal : unit meth
+
+  method oncancel : ('self t, event t) event_listener prop
+
+  method onclose : ('self t, event t) event_listener prop
+end
+
 class type divElement = element
 
 class type paragraphElement = element
@@ -1345,6 +1382,16 @@ class type anchorElement = object
   method _type : js_string t prop
 end
 
+class type detailsElement = object ('self)
+  inherit element
+
+  method open_ : bool t prop
+
+  method name : js_string t prop
+
+  method ontoggle : ('self t, toggleEvent t) event_listener prop
+end
+
 class type imageElement = object ('self)
   inherit element
 
@@ -1845,6 +1892,17 @@ and canvasRenderingContext2D = object
   method arc :
     number_t -> number_t -> number_t -> number_t -> number_t -> bool t -> unit meth
 
+  method ellipse :
+       number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> bool t
+    -> unit meth
+
   method fill : unit meth
 
   method stroke : unit meth
@@ -2526,6 +2584,8 @@ let createDl doc : dListElement t = unsafeCreateElement doc "dl"
 
 let createLi doc : liElement t = unsafeCreateElement doc "li"
 
+let createDialog doc : dialogElement t = unsafeCreateElement doc "dialog"
+
 let createDiv doc : divElement t = unsafeCreateElement doc "div"
 
 let createEmbed doc : embedElement t = unsafeCreateElement doc "embed"
@@ -2696,6 +2756,8 @@ module CoerceTo = struct
 
   let del e = unsafeCoerce "del" e
 
+  let details e = unsafeCoerce "details" e
+
   let div e = unsafeCoerce "div" e
 
   let dl e = unsafeCoerce "dl" e
@@ -3349,6 +3411,7 @@ type taggedElement =
   | Col of tableColElement t
   | Colgroup of tableColElement t
   | Del of modElement t
+  | Dialog of dialogElement t
   | Div of divElement t
   | Dl of dListElement t
   | Embed of embedElement t
@@ -3379,7 +3442,7 @@ type taggedElement =
   | Ol of oListElement t
   | Optgroup of optGroupElement t
   | Option of optionElement t
-  | P of paramElement t
+  | P of paragraphElement t
   | Param of paramElement t
   | Pre of preElement t
   | Q of quoteElement t
