@@ -458,6 +458,14 @@ and clipboardEvent = object
   method clipboardData : dataTransfer t readonly_prop
 end
 
+and toggleEvent = object
+  inherit event
+
+  method newState : js_string t readonly_prop
+
+  method oldState : js_string t readonly_prop
+end
+
 and dataTransfer = object
   method dropEffect : js_string t prop
 
@@ -1114,6 +1122,26 @@ class type dListElement = element
 
 class type liElement = element
 
+class type dialogElement = object
+  inherit element
+
+  method close : unit meth
+
+  method close_returnValue : js_string t -> unit meth
+
+  method open_ : bool t prop
+
+  method returnValue : js_string t prop
+
+  method show : unit meth
+
+  method showModal : unit meth
+
+  method oncancel : ('self t, event t) event_listener prop
+
+  method onclose : ('self t, event t) event_listener prop
+end
+
 class type divElement = element
 
 class type paragraphElement = element
@@ -1170,6 +1198,16 @@ class type anchorElement = object
   method _type : js_string t prop
 end
 
+class type detailsElement = object ('self)
+  inherit element
+
+  method open_ : bool t prop
+
+  method name : js_string t prop
+
+  method ontoggle : ('self t, toggleEvent t) event_listener prop
+end
+
 class type imageElement = object ('self)
   inherit element
 
@@ -1671,6 +1709,17 @@ and canvasRenderingContext2D = object
   method arc :
     number_t -> number_t -> number_t -> number_t -> number_t -> bool t -> unit meth
 
+  method ellipse :
+       number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> number_t
+    -> bool t
+    -> unit meth
+
   method fill : unit meth
 
   method stroke : unit meth
@@ -2286,8 +2335,12 @@ val eventRelatedTarget : #mouseEvent t -> element t opt
 module Event : sig
   type 'a typ = 'a Dom.Event.typ
 
+  val cancel : event t typ
+
   val click : mouseEvent t typ
 
+  val close : event t typ
+
   val copy : clipboardEvent t typ
 
   val cut : clipboardEvent t typ
@@ -2468,6 +2521,8 @@ module Event : sig
 
   val waiting : mediaEvent t typ
 
+  val toggle : toggleEvent t typ
+
   val make : string -> 'a typ
 end
 
@@ -2762,6 +2817,8 @@ val createDl : document t -> dListElement t
 
 val createLi : document t -> liElement t
 
+val createDialog : document t -> dialogElement t
+
 val createDiv : document t -> divElement t
 
 val createEmbed : document t -> embedElement t
@@ -2908,6 +2965,7 @@ type taggedElement =
   | Col of tableColElement t
   | Colgroup of tableColElement t
   | Del of modElement t
+  | Dialog of dialogElement t
   | Div of divElement t
   | Dl of dListElement t
   | Embed of embedElement t
@@ -2938,7 +2996,7 @@ type taggedElement =
   | Ol of oListElement t
   | Optgroup of optGroupElement t
   | Option of optionElement t
-  | P of paramElement t
+  | P of paragraphElement t
   | Param of paramElement t
   | Pre of preElement t
   | Q of quoteElement t
@@ -3010,6 +3068,8 @@ module CoerceTo : sig
 
   val del : #element t -> modElement t opt
 
+  val details : #element t -> detailsElement t opt
+
   val div : #element t -> divElement t opt
 
   val embed : #element t -> embedElement t opt
@@ -3070,7 +3130,7 @@ module CoerceTo : sig
 
   val option : #element t -> optionElement t opt
 
-  val p : #element t -> paramElement t opt
+  val p : #element t -> paragraphElement t opt
 
   val param : #element t -> paramElement t opt
 
