@@ -190,6 +190,24 @@ val click :
   -> #Dom_html.eventTarget Js.t
   -> Dom_html.mouseEvent Js.t Lwt.t
 
+val copy :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.clipboardEvent Js.t Lwt.t
+
+val cut :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.clipboardEvent Js.t Lwt.t
+
+val paste :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.clipboardEvent Js.t Lwt.t
+
 val dblclick :
      ?use_capture:bool
   -> ?passive:bool
@@ -326,7 +344,7 @@ val submit :
      ?use_capture:bool
   -> ?passive:bool
   -> #Dom_html.eventTarget Js.t
-  -> Dom_html.event Js.t Lwt.t
+  -> Dom_html.submitEvent Js.t Lwt.t
 
 val select :
      ?use_capture:bool
@@ -344,6 +362,12 @@ val mousewheel :
     Positive means down or right.
     This interface is compatible with all (recent) browsers. *)
 
+val wheel :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.mousewheelEvent Js.t Lwt.t
+
 val touchstart :
      ?use_capture:bool
   -> ?passive:bool
@@ -428,14 +452,30 @@ val pointerup :
   -> #Dom_html.eventTarget Js.t
   -> Dom_html.pointerEvent Js.t Lwt.t
 
-val transitionend : #Dom_html.eventTarget Js.t -> unit Lwt.t
+val transitionend :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.transitionEvent Js.t Lwt.t
 (** Returns when a CSS transition terminates on the element. *)
 
-val transitionends :
-     ?cancel_handler:bool
+val transitionstart :
+     ?use_capture:bool
+  -> ?passive:bool
   -> #Dom_html.eventTarget Js.t
-  -> (unit Lwt.t -> unit Lwt.t)
-  -> unit Lwt.t
+  -> Dom_html.transitionEvent Js.t Lwt.t
+
+val transitionrun :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.transitionEvent Js.t Lwt.t
+
+val transitioncancel :
+     ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> Dom_html.transitionEvent Js.t Lwt.t
 
 val load :
      ?use_capture:bool
@@ -571,6 +611,30 @@ val clicks :
   -> (Dom_html.mouseEvent Js.t -> unit Lwt.t -> unit Lwt.t)
   -> unit Lwt.t
 
+val copies :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.clipboardEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
+val cuts :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.clipboardEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
+val pastes :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.clipboardEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
 val dblclicks :
      ?cancel_handler:bool
   -> ?use_capture:bool
@@ -731,6 +795,14 @@ val mousewheels :
   -> (Dom_html.mouseEvent Js.t * (int * int) -> unit Lwt.t -> unit Lwt.t)
   -> unit Lwt.t
 
+val wheels :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.mousewheelEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
 val touchstarts :
      ?cancel_handler:bool
   -> ?use_capture:bool
@@ -792,7 +864,7 @@ val submits :
   -> ?use_capture:bool
   -> ?passive:bool
   -> #Dom_html.eventTarget Js.t
-  -> (Dom_html.event Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> (Dom_html.submitEvent Js.t -> unit Lwt.t -> unit Lwt.t)
   -> unit Lwt.t
 
 val selects :
@@ -1051,6 +1123,38 @@ val pointerups :
   -> (Dom_html.pointerEvent Js.t -> unit Lwt.t -> unit Lwt.t)
   -> unit Lwt.t
 
+val transitionends :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.transitionEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
+val transitionstarts :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.transitionEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
+val transitionruns :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.transitionEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
+val transitioncancels :
+     ?cancel_handler:bool
+  -> ?use_capture:bool
+  -> ?passive:bool
+  -> #Dom_html.eventTarget Js.t
+  -> (Dom_html.transitionEvent Js.t -> unit Lwt.t -> unit Lwt.t)
+  -> unit Lwt.t
+
 val request_animation_frame : unit -> unit Lwt.t
 (** Returns when a repaint of the window by the browser starts.
     (see JS method [window.requestAnimationFrame]) *)
