@@ -48,7 +48,7 @@ let parse_lid s =
         match i, c with
         | 0, ('a' .. 'z' | '_') -> ()
         | 0, _ -> assert false
-        | _, ('a' .. 'z' | 'A' .. 'Z' | '_' | '0' .. '9') -> ()
+        | _, ('a' .. 'z' | 'A' .. 'Z' | '_' | '\'' | '0' .. '9') -> ()
         | _ -> assert false)
   in
   let assert_uid =
@@ -56,7 +56,7 @@ let parse_lid s =
         match i, c with
         | 0, 'A' .. 'Z' -> ()
         | 0, _ -> assert false
-        | _, ('a' .. 'z' | 'A' .. 'Z' | '_' | '0' .. '9') -> ()
+        | _, ('a' .. 'z' | 'A' .. 'Z' | '_' | '\'' | '0' .. '9') -> ()
         | _ -> assert false)
   in
   let rec check = function
@@ -245,8 +245,6 @@ let seqlist = function
 
 let check_record_fields =
   List.iter ~f:(function
-      | { Parsetree.pld_mutable = Mutable; _ } ->
-          Location.raise_errorf "%s cannot be derived for mutable records" deriver
       | { pld_type = { ptyp_desc = Ptyp_poly _; _ }; _ } ->
           Location.raise_errorf "%s cannot be derived for polymorphic records" deriver
       | _ -> ())
@@ -499,7 +497,7 @@ and read_body_of_type ?decl y =
   | { Parsetree.ptyp_desc = Ptyp_variant (l, _, _); ptyp_loc = loc; _ } ->
       let e =
         match decl with
-        | Some decl ->
+        | Some ({ ptype_manifest = Some typ; _ } as decl) when typ = y ->
             let e = suffix_decl decl ~suffix:"of_json_with_tag"
             and l =
               let { Parsetree.ptype_params = l; _ } = decl
@@ -507,7 +505,7 @@ and read_body_of_type ?decl y =
               List.map ~f l
             in
             app e l
-        | None -> read_of_poly_variant l y ~loc
+        | Some _ | None -> read_of_poly_variant l y ~loc
       and tag = [%expr [%e lexer "read_vcase"] buf] in
       [%expr [%e e] buf [%e tag]]
   | { Parsetree.ptyp_desc = Ptyp_var v; _ } when poly ->
@@ -775,8 +773,7 @@ let sigs_of_decl ({ Parsetree.ptype_loc; _ } as d) =
   | { Parsetree.ptype_manifest =
         Some { Parsetree.ptyp_desc = Parsetree.Ptyp_variant _; _ }
     ; _
-    } ->
-      read_with_tag_sig_of_decl d :: recognize_sig_of_decl d :: l
+    } -> read_with_tag_sig_of_decl d :: recognize_sig_of_decl d :: l
   | _ -> l
 
 let core_type_exp ({ Parsetree.ptyp_loc; _ } as y) =
